@@ -33,17 +33,37 @@ namespace simgear
 namespace canvas
 {
   /**
-   * Create an canvas Element of type T
+   * Add canvas Element type to factory map
    */
-  template<typename T>
-  ElementPtr createElement( const CanvasWeakPtr& canvas,
-                            const SGPropertyNode_ptr& node,
-                            const Style& style,
-                            Element* parent )
+  template<typename ElementType>
+  void add(ElementFactories& factories)
   {
-    ElementPtr el( new T(canvas, node, style, parent) );
-    el->setSelf(el);
-    return el;
+    ElementType::staticInit();
+    factories[ElementType::TYPE_NAME] = &Element::create<ElementType>;
+  }
+
+  //----------------------------------------------------------------------------
+  ElementFactories Group::_child_factories;
+  const std::string Group::TYPE_NAME = "group";
+
+  void warnTransformExpired(const char* member_name)
+  {
+    SG_LOG( SG_GENERAL,
+            SG_WARN,
+            "canvas::Group::" << member_name << ": Group has expired." );
+  }
+
+  //----------------------------------------------------------------------------
+  void Group::staticInit()
+  {
+    if( isInit<Group>() )
+      return;
+
+    add<Group>(_child_factories);
+    add<Image>(_child_factories);
+    add<Map  >(_child_factories);
+    add<Path >(_child_factories);
+    add<Text >(_child_factories);
   }
 
   //----------------------------------------------------------------------------
@@ -53,11 +73,7 @@ namespace canvas
                 Element* parent ):
     Element(canvas, node, parent_style, parent)
   {
-    _child_factories["group"] = &createElement<Group>;
-    _child_factories["image"] = &createElement<Image>;
-    _child_factories["map"  ] = &createElement<Map  >;
-    _child_factories["path" ] = &createElement<Path >;
-    _child_factories["text" ] = &createElement<Text >;
+    staticInit();
   }
 
   //----------------------------------------------------------------------------
@@ -80,22 +96,54 @@ namespace canvas
   //----------------------------------------------------------------------------
   ElementPtr Group::getChild(const SGPropertyNode* node)
   {
-    ChildList::iterator child = findChild(node);
-    if( child == _children.end() )
-      return ElementPtr();
+    return findChild(node, "");
+  }
 
-    return child->second;
+  //----------------------------------------------------------------------------
+  ElementPtr Group::getChild(const std::string& id)
+  {
+    return findChild(0, id);
+  }
+
+  //----------------------------------------------------------------------------
+  ElementPtr Group::getOrCreateChild( const std::string& type,
+                                      const std::string& id )
+  {
+    ElementPtr child = getChild(id);
+    if( child )
+    {
+      if( child->getProps()->getNameString() == type )
+        return child;
+
+      SG_LOG
+      (
+        SG_GENERAL,
+        SG_WARN,
+        "Group::getOrCreateChild: type missmatch! "
+        "('" << type << "' != '" << child->getProps()->getName() << "', "
+        "id = '" << id << "')"
+      );
+
+      return ElementPtr();
+    }
+
+    return createChild(type, id);
   }
 
   //----------------------------------------------------------------------------
   ElementPtr Group::getElementById(const std::string& id)
   {
-    std::vector<GroupPtr> groups;
-
-    BOOST_FOREACH( ChildList::value_type child, _children )
+    if( !_transform.valid() )
     {
-      const ElementPtr& el = child.second;
-      if( el->getProps()->getStringValue("id") == id )
+      warnTransformExpired("getElementById");
+      return ElementPtr();
+    }
+
+    std::vector<GroupPtr> groups;
+    for(size_t i = 0; i < _transform->getNumChildren(); ++i)
+    {
+      const ElementPtr& el = getChildByIndex(i);
+      if( el->get<std::string>("id") == id )
         return el;
 
       GroupPtr group = boost::dynamic_pointer_cast<Group>(el);
@@ -113,11 +161,23 @@ namespace canvas
     return ElementPtr();
   }
 
+  //----------------------------------------------------------------------------
+  void Group::clearEventListener()
+  {
+    if( !_transform.valid() )
+      return warnTransformExpired("clearEventListener");
+
+    for(size_t i = 0; i < _transform->getNumChildren(); ++i)
+      getChildByIndex(i)->clearEventListener();
+
+    Element::clearEventListener();
+  }
+
   //----------------------------------------------------------------------------
   void Group::update(double dt)
   {
-    BOOST_FOREACH( ChildList::value_type child, _children )
-      child.second->update(dt);
+    for(size_t i = 0; i < _transform->getNumChildren(); ++i)
+      getChildByIndex(i)->update(dt);
 
     Element::update(dt);
   }
@@ -126,30 +186,32 @@ namespace canvas
   bool Group::traverse(EventVisitor& visitor)
   {
     // Iterate in reverse order as last child is displayed on top
-    BOOST_REVERSE_FOREACH( ChildList::value_type child, _children )
+    for(size_t i = _transform->getNumChildren(); i --> 0;)
     {
-      if( child.second->accept(visitor) )
+      if( getChildByIndex(i)->accept(visitor) )
         return true;
     }
     return false;
   }
 
   //----------------------------------------------------------------------------
-  bool Group::setStyle(const SGPropertyNode* style)
+  bool Group::setStyle( const SGPropertyNode* style,
+                        const StyleInfo* style_info )
   {
-    // Don't propagate styles directly applicable to this group
-    if( Element::setStyle(style) )
-      return true;
-
-    if(    style->getParent() != _node
-        && _style.find(style->getNameString()) != _style.end() )
+    if( !canApplyStyle(style) )
       return false;
 
-    bool handled = false;
-    BOOST_FOREACH( ChildList::value_type child, _children )
+    bool handled = setStyleImpl(style, style_info);
+    if( style_info->inheritable )
     {
-      if( child.second->setStyle(style) )
-        handled = true;
+      if( !_transform.valid() )
+      {
+        warnTransformExpired("setStyle");
+        return false;
+      }
+
+      for(size_t i = 0; i < _transform->getNumChildren(); ++i)
+        handled |= getChildByIndex(i)->setStyle(style, style_info);
     }
 
     return handled;
@@ -159,17 +221,23 @@ namespace canvas
   osg::BoundingBox Group::getTransformedBounds(const osg::Matrix& m) const
   {
     osg::BoundingBox bb;
-
-    BOOST_FOREACH( ChildList::value_type child, _children )
+    if( !_transform.valid() )
     {
-      if( !child.second->getMatrixTransform()->getNodeMask() )
+      warnTransformExpired("getTransformedBounds");
+      return bb;
+    }
+
+    for(size_t i = 0; i < _transform->getNumChildren(); ++i)
+    {
+      const ElementPtr& child = getChildByIndex(i);
+      if( !child->getMatrixTransform()->getNodeMask() )
         continue;
 
       bb.expandBy
       (
-        child.second->getTransformedBounds
+        child->getTransformedBounds
         (
-          child.second->getMatrixTransform()->getMatrix() * m
+          child->getMatrixTransform()->getMatrix() * m
         )
       );
     }
@@ -177,31 +245,42 @@ namespace canvas
     return bb;
   }
 
+  //----------------------------------------------------------------------------
+  ElementFactory Group::getChildFactory(const std::string& type) const
+  {
+    ElementFactories::iterator child_factory = _child_factories.find(type);
+    if( child_factory != _child_factories.end() )
+      return child_factory->second;
+
+    return ElementFactory();
+  }
+
   //----------------------------------------------------------------------------
   void Group::childAdded(SGPropertyNode* child)
   {
     if( child->getParent() != _node )
       return;
 
-    ChildFactories::iterator child_factory =
-      _child_factories.find( child->getNameString() );
-    if( child_factory != _child_factories.end() )
+    ElementFactory child_factory = getChildFactory( child->getNameString() );
+    if( child_factory )
     {
-      ElementPtr element = child_factory->second(_canvas, child, _style, this);
+      if( !_transform.valid() )
+        return warnTransformExpired("childAdded");
+
+      ElementPtr element = child_factory(_canvas, child, _style, this);
 
       // Add to osg scene graph...
       _transform->addChild( element->getMatrixTransform() );
-      _children.push_back( ChildList::value_type(child, element) );
+
+      // ...and ensure correct ordering
+      handleZIndexChanged(element);
 
       return;
     }
 
-    if( !Element::setStyle(child) )
-    {
-      // Only add style if not applicable to group itself
+    StyleInfo const* style = getStyleInfo(child->getNameString());
+    if( style && style->inheritable )
       _style[ child->getNameString() ] = child;
-      setStyle(child);
-    }
   }
 
   //----------------------------------------------------------------------------
@@ -210,10 +289,15 @@ namespace canvas
     if( node->getParent() != _node )
       return;
 
-    if( _child_factories.find(node->getNameString()) != _child_factories.end() )
+    if( getChildFactory(node->getNameString()) )
     {
-      ChildList::iterator child = findChild(node);
-      if( child == _children.end() )
+      if( !_transform.valid() )
+        // If transform is destroyed also all children are destroyed, so we can
+        // not do anything here.
+        return;
+
+      ElementPtr child = getChild(node);
+      if( !child )
         SG_LOG
         (
           SG_GL,
@@ -222,15 +306,14 @@ namespace canvas
         );
       else
       {
-        _transform->removeChild( child->second->getMatrixTransform() );
-        _children.erase(child);
+        // Remove child from the scenegraph (this automatically invalidates the
+        // reference on the element hold by the MatrixTransform)
+        child->onDestroy();
       }
     }
     else
     {
-      Style::iterator style = _style.find(node->getNameString());
-      if( style != _style.end() )
-        _style.erase(style);
+      _style.erase( node->getNameString() );
     }
   }
 
@@ -239,54 +322,52 @@ namespace canvas
   {
     if(    node->getParent()->getParent() == _node
         && node->getNameString() == "z-index" )
-      return handleZIndexChanged(node->getParent(), node->getIntValue());
+      return handleZIndexChanged( getChild(node->getParent()),
+                                  node->getIntValue() );
   }
 
   //----------------------------------------------------------------------------
-  void Group::handleZIndexChanged(SGPropertyNode* node, int z_index)
+  void Group::handleZIndexChanged(ElementPtr child, int z_index)
   {
-    ChildList::iterator child = findChild(node);
-    if( child == _children.end() )
+    if( !child || !_transform.valid() )
       return;
 
-    osg::Node* tf = child->second->getMatrixTransform();
-    int index = _transform->getChildIndex(tf),
-        index_new = index;
+    osg::ref_ptr<osg::MatrixTransform> tf = child->getMatrixTransform();
+    size_t index = _transform->getChildIndex(tf),
+           index_new = index;
 
-    ChildList::iterator next = child;
-    ++next;
-
-    while(    next != _children.end()
-           && next->first->getIntValue("z-index", 0) <= z_index )
+    for(;; ++index_new)
     {
-      ++index_new;
-      ++next;
+      if( index_new + 1 == _transform->getNumChildren() )
+        break;
+
+      // Move to end of block with same index (= move upwards until the next
+      // element has a higher index)
+      if( getChildByIndex(index_new + 1)->get<int>("z-index", 0) > z_index )
+        break;
     }
 
-    if( index_new != index )
+    if( index_new == index )
     {
-      _children.insert(next, *child);
-    }
-    else
-    {
-      ChildList::iterator prev = child;
-      while(    prev != _children.begin()
-             && (--prev)->first->getIntValue("z-index", 0) > z_index)
+      // We were not able to move upwards so now try downwards
+      for(;; --index_new)
       {
-        --index_new;
+        if( index_new == 0 )
+          break;
+
+        // Move to end of block with same index (= move downwards until the
+        // previous element has the same or a lower index)
+        if( getChildByIndex(index_new - 1)->get<int>("z-index", 0) <= z_index )
+          break;
       }
 
       if( index == index_new )
         return;
-
-      _children.insert(prev, *child);
     }
 
     _transform->removeChild(index);
     _transform->insertChild(index_new, tf);
 
-    _children.erase(child);
-
     SG_LOG
     (
       SG_GENERAL,
@@ -296,14 +377,42 @@ namespace canvas
   }
 
   //----------------------------------------------------------------------------
-  Group::ChildList::iterator Group::findChild(const SGPropertyNode* node)
+  ElementPtr Group::getChildByIndex(size_t index) const
   {
-    return std::find_if
-    (
-      _children.begin(),
-      _children.end(),
-      boost::bind(&ChildList::value_type::first, _1) == node
-    );
+    assert(_transform.valid());
+    OSGUserData* ud =
+      static_cast<OSGUserData*>(_transform->getChild(index)->getUserData());
+    assert(ud);
+    return ud->element;
+  }
+
+  //----------------------------------------------------------------------------
+  ElementPtr Group::findChild( const SGPropertyNode* node,
+                               const std::string& id ) const
+  {
+    if( !_transform.valid() )
+    {
+      warnTransformExpired("findChild");
+      return ElementPtr();
+    }
+
+    for(size_t i = 0; i < _transform->getNumChildren(); ++i)
+    {
+      ElementPtr el = getChildByIndex(i);
+
+      if( node )
+      {
+        if( el->getProps() == node )
+          return el;
+      }
+      else
+      {
+        if( el->get<std::string>("id") == id )
+          return el;
+      }
+    }
+
+    return ElementPtr();
   }
 
 } // namespace canvas
