@@ -629,7 +629,7 @@ bool SGMetar::scanWeather()
 	weather = pre + weather + post;
 	weather.erase(weather.length() - 1);
 	_weather.push_back(weather);
-    if( w.phenomena.size() > 0 )
+    if( ! w.phenomena.empty() )
         _weather2.push_back( w );
 	_grpcount++;
 	return true;
