@@ -26,9 +26,11 @@
 
 #include <simgear_config.h>
 #include <simgear/debug/logstream.hxx>
+#include <simgear/misc/strutils.hxx>
 #include <stdio.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <fstream>
 
 #ifdef _WIN32
 #  include <direct.h>
@@ -38,6 +40,7 @@
 #include <boost/algorithm/string/case_conv.hpp>
 
 using std::string;
+using simgear::strutils::starts_with;
 
 /**
  * define directory path separators
@@ -52,6 +55,34 @@ static const char sgSearchPathSep = ';';
 static const char sgSearchPathSep = ':';
 #endif
 
+#ifdef _WIN32
+
+#include <ShlObj.h> // for CSIDL
+
+static SGPath pathForCSIDL(int csidl, const SGPath::PermissonChecker& checker)
+{
+	typedef BOOL (WINAPI*GetSpecialFolderPath)(HWND, LPSTR, int, BOOL);
+	static GetSpecialFolderPath SHGetSpecialFolderPath = NULL;
+
+	// lazy open+resolve of shell32
+	if (!SHGetSpecialFolderPath) {
+		HINSTANCE shellDll = ::LoadLibrary("shell32");
+		SHGetSpecialFolderPath = (GetSpecialFolderPath) GetProcAddress(shellDll, "SHGetSpecialFolderPathA");
+	}
+
+	if (!SHGetSpecialFolderPath){
+		return SGPath();
+	}
+
+	char path[MAX_PATH];
+	if (SHGetSpecialFolderPath(0, path, csidl, false)) {
+		return SGPath(path, checker);
+	}
+
+	return SGPath();
+}
+
+#endif
 
 // For windows, replace "\" by "/".
 void
@@ -72,27 +103,35 @@ SGPath::fix()
 
 
 // default constructor
-SGPath::SGPath()
+SGPath::SGPath(PermissonChecker validator)
     : path(""),
+    _permission_checker(validator),
     _cached(false),
+    _rwCached(false),
     _cacheEnabled(true)
 {
 }
 
 
 // create a path based on "path"
-SGPath::SGPath( const std::string& p )
+SGPath::SGPath( const std::string& p, PermissonChecker validator )
     : path(p),
+    _permission_checker(validator),
     _cached(false),
+    _rwCached(false),
     _cacheEnabled(true)
 {
     fix();
 }
 
 // create a path based on "path" and a "subpath"
-SGPath::SGPath( const SGPath& p, const std::string& r )
+SGPath::SGPath( const SGPath& p,
+                const std::string& r,
+                PermissonChecker validator )
     : path(p.path),
+    _permission_checker(validator),
     _cached(false),
+    _rwCached(false),
     _cacheEnabled(p._cacheEnabled)
 {
     append(r);
@@ -101,8 +140,12 @@ SGPath::SGPath( const SGPath& p, const std::string& r )
 
 SGPath::SGPath(const SGPath& p) :
   path(p.path),
+  _permission_checker(p._permission_checker),
   _cached(p._cached),
+  _rwCached(p._rwCached),
   _cacheEnabled(p._cacheEnabled),
+  _canRead(p._canRead),
+  _canWrite(p._canWrite),
   _exists(p._exists),
   _isDir(p._isDir),
   _isFile(p._isFile),
@@ -113,8 +156,12 @@ SGPath::SGPath(const SGPath& p) :
 SGPath& SGPath::operator=(const SGPath& p)
 {
   path = p.path;
+  _permission_checker = p._permission_checker,
   _cached = p._cached;
+  _rwCached = p._rwCached;
   _cacheEnabled = p._cacheEnabled;
+  _canRead = p._canRead;
+  _canWrite = p._canWrite;
   _exists = p._exists;
   _isDir = p._isDir;
   _isFile = p._isFile;
@@ -132,16 +179,31 @@ void SGPath::set( const string& p ) {
     path = p;
     fix();
     _cached = false;
+    _rwCached = false;
 }
 
+//------------------------------------------------------------------------------
+void SGPath::setPermissonChecker(PermissonChecker validator)
+{
+  _permission_checker = validator;
+  _rwCached = false;
+}
+
+//------------------------------------------------------------------------------
+SGPath::PermissonChecker SGPath::getPermissonChecker() const
+{
+  return _permission_checker;
+}
+
+//------------------------------------------------------------------------------
 void SGPath::set_cached(bool cached)
 {
-    _cacheEnabled = cached;
+  _cacheEnabled = cached;
 }
 
 // append another piece to the existing path
 void SGPath::append( const string& p ) {
-    if ( path.size() == 0 ) {
+    if ( path.empty() ) {
         path = p;
     } else {
     if ( p[0] != sgDirPathSep ) {
@@ -151,6 +213,15 @@ void SGPath::append( const string& p ) {
     }
     fix();
     _cached = false;
+    _rwCached = false;
+}
+
+//------------------------------------------------------------------------------
+SGPath SGPath::operator/( const std::string& p ) const
+{
+  SGPath ret = *this;
+  ret.append(p);
+  return ret;
 }
 
 //add a new path component to the existing path string
@@ -162,13 +233,14 @@ void SGPath::add( const string& p ) {
 // concatenate a string to the end of the path without inserting a
 // path separator
 void SGPath::concat( const string& p ) {
-    if ( path.size() == 0 ) {
+    if ( path.empty() ) {
         path = p;
     } else {
         path += p;
     }
     fix();
     _cached = false;
+    _rwCached = false;
 }
 
 
@@ -267,14 +339,21 @@ void SGPath::validate() const
   if (_cached && _cacheEnabled) {
     return;
   }
-  
+
+  if (path.empty()) {
+	  _exists = false;
+	  return;
+  }
+
 #ifdef _WIN32
   struct _stat buf ;
-
   bool remove_trailing = false;
-  if ( path.length() > 1 && path[path.length()-1] == '/' )
-      remove_trailing=true;
-  if (_stat (path.substr(0,remove_trailing?path.length()-1:path.length()).c_str(), &buf ) < 0) {
+  string statPath(path);
+  if ((path.length() > 1) && (path.back() == '/')) {
+	  statPath.pop_back();
+  }
+      
+  if (_stat(statPath.c_str(), &buf ) < 0) {
     _exists = false;
   } else {
     _exists = true;
@@ -299,12 +378,46 @@ void SGPath::validate() const
   _cached = true;
 }
 
+void SGPath::checkAccess() const
+{
+  if( _rwCached && _cacheEnabled )
+    return;
+
+  if( _permission_checker )
+  {
+    Permissions p = _permission_checker(*this);
+    _canRead = p.read;
+    _canWrite = p.write;
+  }
+  else
+  {
+    _canRead = true;
+    _canWrite = true;
+  }
+
+  _rwCached = true;
+}
+
 bool SGPath::exists() const
 {
   validate();
   return _exists;
 }
 
+//------------------------------------------------------------------------------
+bool SGPath::canRead() const
+{
+  checkAccess();
+  return _canRead;
+}
+
+//------------------------------------------------------------------------------
+bool SGPath::canWrite() const
+{
+  checkAccess();
+  return _canWrite;
+}
+
 bool SGPath::isDir() const
 {
   validate();
@@ -333,7 +446,7 @@ int SGPath::create_dir( mode_t mode ) {
     bool absolute = !path.empty() && path[0] == sgDirPathSep;
 
     unsigned int i = 1;
-    SGPath dir = absolute ? string( 1, sgDirPathSep ) : "";
+    SGPath dir(absolute ? string( 1, sgDirPathSep ) : "", _permission_checker);
     dir.concat( path_elements[0] );
 #ifdef _WIN32
     if ( dir.str().find(':') != string::npos && path_elements.size() >= 2 ) {
@@ -349,16 +462,26 @@ int SGPath::create_dir( mode_t mode ) {
     if ( r == 0 ) {
         return 0; // Directory already exists
     }
-    if ( sgMkDir( dir.c_str(), mode) ) {
-        SG_LOG( SG_IO, SG_ALERT, "Error creating directory: " + dir.str() );
+    for(;;)
+    {
+      if( !dir.canWrite() )
+      {
+        SG_LOG( SG_IO,
+                SG_WARN, "Error creating directory: (" << dir.str() << ")" <<
+                                                  " reason: access denied" );
+        return -3;
+      }
+      else if( sgMkDir(dir.c_str(), mode) )
+      {
+        SG_LOG( SG_IO,
+                SG_ALERT, "Error creating directory: (" << dir.str() << ")" );
         return -2;
-    }
-    for(; i < path_elements.size(); i++) {
-        dir.append(path_elements[i]);
-        if ( sgMkDir( dir.c_str(), mode) ) {
-            SG_LOG( SG_IO, SG_ALERT, "Error creating directory: " + dir.str() );
-            return -2;
-        }
+      }
+
+      if( i >= path_elements.size() )
+        return  0;
+
+      dir.append(path_elements[i++]);
     }
 
     return 0;
@@ -367,7 +490,7 @@ int SGPath::create_dir( mode_t mode ) {
 string_list sgPathBranchSplit( const string &dirpath ) {
     string_list path_elements;
     string element, path = dirpath;
-    while ( path.size() ) {
+    while ( ! path.empty() ) {
         size_t p = path.find( sgDirPathSep );
         if ( p != string::npos ) {
             element = path.substr( 0, p );
@@ -376,7 +499,7 @@ string_list sgPathBranchSplit( const string &dirpath ) {
             element = path;
             path = "";
         }
-        if ( element.size() )
+        if ( ! element.empty() )
             path_elements.push_back( element );
     }
     return path_elements;
@@ -425,7 +548,7 @@ bool SGPath::isAbsolute() const
 
 bool SGPath::isNull() const
 {
-  return path.empty() || (path == "");
+  return path.empty();
 }
 
 std::string SGPath::str_native() const
@@ -445,13 +568,27 @@ std::string SGPath::str_native() const
 #endif
 }
 
+//------------------------------------------------------------------------------
 bool SGPath::remove()
 {
-    int err = ::unlink(c_str());
-    if (err) {
-        SG_LOG(SG_IO, SG_WARN,  "file remove failed: (" << str() << ") " << strerror(errno));
-    }
-    return (err == 0);
+  if( !canWrite() )
+  {
+    SG_LOG( SG_IO, SG_WARN, "file remove failed: (" << str() << ")"
+                                               " reason: access denied" );
+    return false;
+  }
+
+  int err = ::unlink(c_str());
+  if( err )
+  {
+    SG_LOG( SG_IO, SG_WARN, "file remove failed: (" << str() << ") "
+                                               " reason: " << strerror(errno) );
+    // TODO check if failed unlink can really change any of the cached values
+  }
+
+  _cached = false; // stat again if required
+  _rwCached = false;
+  return (err == 0);
 }
 
 time_t SGPath::modTime() const
@@ -470,19 +607,148 @@ bool SGPath::operator!=(const SGPath& other) const
     return (path != other.path);
 }
 
+//------------------------------------------------------------------------------
 bool SGPath::rename(const SGPath& newName)
 {
-    if (::rename(c_str(), newName.c_str()) != 0) {
-        SG_LOG(SG_IO, SG_WARN, "renamed failed: from " << str() << " to " << newName.str()
-            << " reason: " << strerror(errno));
-        return false;
-    }
-    
-    path = newName.path;
-    _cached = false;
-    return true;
+  if( !canRead() || !canWrite() || !newName.canWrite() )
+  {
+    SG_LOG( SG_IO, SG_WARN, "rename failed: from " << str() <<
+                                            " to " << newName.str() <<
+                                            " reason: access denied" );
+    return false;
+  }
+
+#ifdef SG_WINDOWS
+	if (newName.exists()) {
+		SGPath r(newName);
+		if (!r.remove()) {
+			return false;
+		}
+	}
+#endif
+  if( ::rename(c_str(), newName.c_str()) != 0 )
+  {
+    SG_LOG( SG_IO, SG_WARN, "rename failed: from " << str() <<
+                                            " to " << newName.str() <<
+                                            " reason: " << strerror(errno) );
+    return false;
+  }
+
+  path = newName.path;
+
+  // Do not remove permission checker (could happen for example if just using
+  // a std::string as new name)
+  if( newName._permission_checker )
+    _permission_checker = newName._permission_checker;
+
+  _cached = false;
+  _rwCached = false;
+
+  return true;
 }
 
+//------------------------------------------------------------------------------
+SGPath SGPath::fromEnv(const char* name, const SGPath& def)
+{
+  const char* val = getenv(name);
+  if( val && val[0] )
+    return SGPath(val, def._permission_checker);
+  return def;
+}
+
+#ifdef _WIN32
+//------------------------------------------------------------------------------
+SGPath SGPath::home(const SGPath& def)
+{
+  // TODO
+  return def;
+}
+#else
+//------------------------------------------------------------------------------
+SGPath SGPath::home(const SGPath& def)
+{
+  return fromEnv("HOME", def);
+}
+#endif
+
+#ifdef _WIN32
+//------------------------------------------------------------------------------
+SGPath SGPath::desktop(const SGPath& def)
+{
+	SGPath r = pathForCSIDL(CSIDL_DESKTOPDIRECTORY, def._permission_checker);
+	if (!r.isNull()) {
+		return r;
+	}
+
+	SG_LOG(SG_GENERAL, SG_ALERT, "SGPath::desktop() failed, bad" );
+	return def;
+}
+
+SGPath SGPath::documents(const SGPath& def)
+{
+	SGPath r = pathForCSIDL(CSIDL_MYDOCUMENTS, def._permission_checker);
+	if (!r.isNull()) {
+		return r;
+	}
+
+	SG_LOG(SG_GENERAL, SG_ALERT, "SGPath::documents() failed, bad" );
+	return def;
+}
+#elif __APPLE__
+#include <CoreServices/CoreServices.h>
+
+//------------------------------------------------------------------------------
+SGPath SGPath::desktop(const SGPath& def)
+{
+  FSRef ref;
+  OSErr err = FSFindFolder(kUserDomain, kDesktopFolderType, false, &ref);
+  if (err)
+    return def;
+
+  unsigned char path[1024];
+  if (FSRefMakePath(&ref, path, 1024) != noErr)
+      return def;
+
+  return SGPath((const char*) path, def._permission_checker);
+}
+#else
+//------------------------------------------------------------------------------
+SGPath SGPath::desktop(const SGPath& def)
+{
+  // http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html
+
+  // $XDG_CONFIG_HOME defines the base directory relative to which user specific
+  // configuration files should be stored. If $XDG_CONFIG_HOME is either not set
+  // or empty, a default equal to $HOME/.config should be used.
+  const SGPath user_dirs = fromEnv("XDG_CONFIG_HOME", home() / ".config")
+                         / "user-dirs.dirs";
+
+  // Format is XDG_xxx_DIR="$HOME/yyy", where yyy is a shell-escaped
+  // homedir-relative path, or XDG_xxx_DIR="/yyy", where /yyy is an absolute
+  // path. No other format is supported.
+  const std::string DESKTOP = "XDG_DESKTOP_DIR=\"";
+
+  std::ifstream user_dirs_file( user_dirs.c_str() );
+  std::string line;
+  while( std::getline(user_dirs_file, line).good() )
+  {
+    if( !starts_with(line, DESKTOP) || *line.rbegin() != '"' )
+      continue;
+
+    // Extract dir from XDG_DESKTOP_DIR="<dir>"
+    line = line.substr(DESKTOP.length(), line.length() - DESKTOP.length() - 1 );
+
+    const std::string HOME = "$HOME";
+    if( starts_with(line, HOME) )
+      return home(def) / simgear::strutils::unescape(line.substr(HOME.length()));
+
+    return SGPath(line, def._permission_checker);
+  }
+
+  return home(def) / "Desktop";
+}
+#endif
+
 std::string SGPath::realpath() const
 {
 #if (defined(__APPLE__) && MAC_OS_X_VERSION_MAX_ALLOWED <= 1050)
@@ -490,7 +756,7 @@ std::string SGPath::realpath() const
     // This means relative paths cannot be used on Mac OS <= 10.5
     return path;
 #else
-  #if defined(_MSC_VER)
+  #if defined(_MSC_VER) /*for MS compilers */ || defined(_WIN32) /*needed for non MS windows compilers like MingW*/
     // with absPath NULL, will allocate, and ignore length
     char *buf = _fullpath( NULL, path.c_str(), _MAX_PATH );
   #else
