@@ -138,7 +138,8 @@ PropsVisitor::endXML ()
  * Check a yes/no flag, with default.
  */
 static bool
-checkFlag (const char * flag, bool defaultState = true)
+checkFlag (const char * flag, const sg_location& location,
+           bool defaultState = true)
 {
   if (flag == 0)
     return defaultState;
@@ -151,7 +152,7 @@ checkFlag (const char * flag, bool defaultState = true)
     message += flag;
     message += '\'';
 				// FIXME: add location info
-    throw sg_io_exception(message, "SimGear Property Reader");
+    throw sg_io_exception(message, location, "SimGear Property Reader");
   }
 }
 
@@ -159,13 +160,14 @@ void
 PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
 {
   const char * attval;
+  const sg_location location(getPath(), getLine(), getColumn());
 
   if (_level == 0) {
     if (strcmp(name, "PropertyList")) {
       string message = "Root element name is ";
       message += name;
       message += "; expected PropertyList";
-      throw sg_io_exception(message, "SimGear Property Reader");
+      throw sg_io_exception(message, location, "SimGear Property Reader");
     }
 
 				// Check for an include.
@@ -175,7 +177,10 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
           SGPath path = simgear::ResourceManager::instance()->findPath(attval, SGPath(_base).dir());
           if (path.isNull())
           {
-              throw sg_io_exception("Cannot open file", sg_location(attval));
+              string message ="Cannot open file ";
+              message += attval;
+              throw sg_io_exception(message, location,
+                                    "SimGear Property Reader");
           }
           readProperties(path.str(), _root, 0, _extended);
       } catch (sg_io_exception &e) {
@@ -204,7 +209,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
     SGPropertyNode * node = st.node->getChild(strName, index, true);
     if (!node->getAttribute(SGPropertyNode::WRITE)) {
       SG_LOG(SG_INPUT, SG_ALERT, "Not overwriting write-protected property "
-          << node->getPath(true));
+             << node->getPath(true) << "\n at " << location.asString());
       node = &null;
     }
 
@@ -214,32 +219,33 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
     int mode = _default_mode;
 
     attval = atts.getValue("read");
-    if (checkFlag(attval, true))
+    if (checkFlag(attval, location, true))
       mode |= SGPropertyNode::READ;
     attval = atts.getValue("write");
-    if (checkFlag(attval, true))
+    if (checkFlag(attval, location, true))
       mode |= SGPropertyNode::WRITE;
     attval = atts.getValue("archive");
-    if (checkFlag(attval, false))
+    if (checkFlag(attval, location, false))
       mode |= SGPropertyNode::ARCHIVE;
     attval = atts.getValue("trace-read");
-    if (checkFlag(attval, false))
+    if (checkFlag(attval, location, false))
       mode |= SGPropertyNode::TRACE_READ;
     attval = atts.getValue("trace-write");
-    if (checkFlag(attval, false))
+    if (checkFlag(attval, location, false))
       mode |= SGPropertyNode::TRACE_WRITE;
     attval = atts.getValue("userarchive");
-    if (checkFlag(attval, false))
+    if (checkFlag(attval, location, false))
       mode |= SGPropertyNode::USERARCHIVE;
     attval = atts.getValue("preserve");
-    if (checkFlag(attval, false))
+    if (checkFlag(attval, location, false))
       mode |= SGPropertyNode::PRESERVE;
 
 				// Check for an alias.
     attval = atts.getValue("alias");
     if (attval != 0) {
       if (!node->alias(attval))
-	SG_LOG(SG_INPUT, SG_ALERT, "Failed to set alias to " << attval);
+	SG_LOG(SG_INPUT, SG_ALERT, "Failed to set alias to " << attval
+               << "\n at " << location.asString());
     }
 
 				// Check for an include.
@@ -250,7 +256,10 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
           SGPath path = simgear::ResourceManager::instance()->findPath(attval, SGPath(_base).dir());
           if (path.isNull())
           {
-              throw sg_io_exception("Cannot open file", sg_location(attval));
+              string message ="Cannot open file ";
+              message += attval;
+              throw sg_io_exception(message, location,
+                                    "SimGear Property Reader");
           }
           readProperties(path.str(), node, 0, _extended);
       } catch (sg_io_exception &e) {
@@ -258,7 +267,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
       }
 
       attval = atts.getValue("omit-node");
-      omit = checkFlag(attval, false);
+      omit = checkFlag(attval, location, false);
     }
 
     const char *type = atts.getValue("type");
@@ -276,6 +285,7 @@ PropsVisitor::endElement (const char * name)
 {
   State &st = state();
   bool ret;
+  const sg_location location(getPath(), getLine(), getColumn());
 
 				// If there are no children and it's
 				// not an alias, then it's a leaf value.
@@ -310,12 +320,13 @@ PropsVisitor::endElement (const char * name)
       message += st.type;
       message += '\'';
 				// FIXME: add location information
-      throw sg_io_exception(message, "SimGear Property Reader");
+      throw sg_io_exception(message, location, "SimGear Property Reader");
     }
     if (!ret)
       SG_LOG(SG_INPUT, SG_ALERT, "readProperties: Failed to set "
 	     << st.node->getPath() << " to value \""
-	     << _data << "\" with type " << st.type);
+	     << _data << "\" with type " << st.type << "\n at "
+             << location.asString());
   }
 
 				// Set the access-mode attributes now,
@@ -605,7 +616,7 @@ writeProperties (const string &file, const SGPropertyNode * start_node,
                  bool write_all, SGPropertyNode::Attribute archive_flag)
 {
   SGPath path(file.c_str());
-  path.create_dir(0777);
+  path.create_dir(0755);
 
   ofstream output(file.c_str());
   if (output.good()) {
@@ -629,6 +640,64 @@ writeProperties (const char* file, const SGPropertyNode * start_node)
 ////////////////////////////////////////////////////////////////////////
 
 
+bool
+copyPropertyValue(const SGPropertyNode *in, SGPropertyNode *out)
+{
+    using namespace simgear;
+    bool retval = true;
+    
+    if (!in->hasValue()) {
+        return true;
+    }
+    
+    switch (in->getType()) {
+        case props::BOOL:
+            if (!out->setBoolValue(in->getBoolValue()))
+                retval = false;
+            break;
+        case props::INT:
+            if (!out->setIntValue(in->getIntValue()))
+                retval = false;
+            break;
+        case props::LONG:
+            if (!out->setLongValue(in->getLongValue()))
+                retval = false;
+            break;
+        case props::FLOAT:
+            if (!out->setFloatValue(in->getFloatValue()))
+                retval = false;
+            break;
+        case props::DOUBLE:
+            if (!out->setDoubleValue(in->getDoubleValue()))
+                retval = false;
+            break;
+        case props::STRING:
+            if (!out->setStringValue(in->getStringValue()))
+                retval = false;
+            break;
+        case props::UNSPECIFIED:
+            if (!out->setUnspecifiedValue(in->getStringValue()))
+                retval = false;
+            break;
+        case props::VEC3D:
+            if (!out->setValue(in->getValue<SGVec3d>()))
+                retval = false;
+            break;
+        case props::VEC4D:
+            if (!out->setValue(in->getValue<SGVec4d>()))
+                retval = false;
+            break;
+        default:
+            if (in->isAlias())
+                break;
+            string message = "Unknown internal SGPropertyNode type";
+            message += in->getType();
+            throw sg_error(message, "SimGear Property Reader");
+    }
+    
+    return retval;
+}
+
 /**
  * Copy one property tree to another.
  * 
@@ -646,57 +715,11 @@ copyProperties (const SGPropertyNode *in, SGPropertyNode *out,
                 int attr_value, int attr_mask)
 {
   using namespace simgear;
-  bool retval = true;
-
-				// First, copy the actual value,
-				// if any.
-  if (in->hasValue()) {
-    switch (in->getType()) {
-    case props::BOOL:
-      if (!out->setBoolValue(in->getBoolValue()))
-	retval = false;
-      break;
-    case props::INT:
-      if (!out->setIntValue(in->getIntValue()))
-	retval = false;
-      break;
-    case props::LONG:
-      if (!out->setLongValue(in->getLongValue()))
-	retval = false;
-      break;
-    case props::FLOAT:
-      if (!out->setFloatValue(in->getFloatValue()))
-	retval = false;
-      break;
-    case props::DOUBLE:
-      if (!out->setDoubleValue(in->getDoubleValue()))
-	retval = false;
-      break;
-    case props::STRING:
-      if (!out->setStringValue(in->getStringValue()))
-	retval = false;
-      break;
-    case props::UNSPECIFIED:
-      if (!out->setUnspecifiedValue(in->getStringValue()))
-	retval = false;
-      break;
-    case props::VEC3D:
-      if (!out->setValue(in->getValue<SGVec3d>()))
-        retval = false;
-      break;
-    case props::VEC4D:
-      if (!out->setValue(in->getValue<SGVec4d>()))
-        retval = false;
-      break;
-    default:
-      if (in->isAlias())
-	break;
-      string message = "Unknown internal SGPropertyNode type";
-      message += in->getType();
-      throw sg_error(message, "SimGear Property Reader");
-    }
+  bool retval = copyPropertyValue(in, out);
+  if (!retval) {
+    return false;
   }
-
+    
   // copy the attributes.
   out->setAttributes( in->getAttributes() );
 
@@ -737,4 +760,41 @@ copyProperties (const SGPropertyNode *in, SGPropertyNode *out,
   return retval;
 }
 
+
+bool
+copyPropertiesWithAttribute(const SGPropertyNode *in, SGPropertyNode *out,
+                             SGPropertyNode::Attribute attr)
+{
+    bool retval = copyPropertyValue(in, out);
+    if (!retval) {
+        return false;
+    }
+    out->setAttributes( in->getAttributes() );
+    
+    // if attribute is set directly on this node, we don't require it on
+    // descendent nodes. (Allows setting an attribute on an entire sub-tree
+    // of nodes)
+    if ((attr != SGPropertyNode::NO_ATTR) && out->getAttribute(attr)) {
+        attr = SGPropertyNode::NO_ATTR;
+    }
+    
+    int nChildren = in->nChildren();
+    for (int i = 0; i < nChildren; i++) {
+        const SGPropertyNode* in_child = in->getChild(i);
+        if ((attr != SGPropertyNode::NO_ATTR) && !isArchivable(in_child, attr))
+            continue;
+        
+         SGPropertyNode* out_child = out->getChild(in_child->getNameString(),
+                                      in_child->getIndex(),
+                                      true);
+        
+        bool ok = copyPropertiesWithAttribute(in_child, out_child, attr);
+        if (!ok) {
+            return false;
+        }
+    }// of children iteration
+    
+    return true;
+}
+
 // end of props_io.cxx
