@@ -118,7 +118,7 @@ void Technique::validateInContext(GraphicsContext* gc)
     Status oldVal = contextInfo.valid();
     Status newVal = INVALID;
     expression::FixedLengthBinding<1> binding;
-    binding.getBindings()[_contextIdLocation].val.intVal = contextId;
+    binding.getBindings()[_contextIdLocation] = expression::Value((int) contextId);
     if (_validExpression->getValue(&binding))
         newVal = VALID;
     contextInfo.valid.compareAndSwap(oldVal, newVal);
@@ -167,12 +167,12 @@ Technique::processDrawables(const EffectGeode::DrawablesIterator& begin,
     BOOST_FOREACH(ref_ptr<Pass>& pass, passes)
     {
         osg::ref_ptr<osg::StateSet> ss = pass;
-        if (ecv && ( pass->getBufferUnitList().size() != 0 || pass->getPositionedUniformMap().size() != 0 ) ) {
+        if (ecv && ( ! pass->getBufferUnitList().empty() || ! pass->getPositionedUniformMap().empty() ) ) {
             ss = static_cast<osg::StateSet*>(
-                pass->clone( osg::CopyOp( ( pass->getBufferUnitList().size() != 0 ?
+                pass->clone( osg::CopyOp( ( ! pass->getBufferUnitList().empty() ?
                                                         osg::CopyOp::DEEP_COPY_TEXTURES :
                                                         osg::CopyOp::SHALLOW_COPY ) |
-                                           ( pass->getPositionedUniformMap().size() != 0 ?
+                                           ( ! pass->getPositionedUniformMap().empty() ?
                                                         osg::CopyOp::DEEP_COPY_UNIFORMS :
                                                         osg::CopyOp::SHALLOW_COPY ) )
                 )
@@ -271,19 +271,19 @@ class ExtensionSupportedExpression
 {
 public:
     ExtensionSupportedExpression() {}
-    ExtensionSupportedExpression(const string& extString)
+    ExtensionSupportedExpression(const std::string& extString)
         : _extString(extString)
     {
     }
-    const string& getExtensionString() { return _extString; }
-    void setExtensionString(const string& extString) { _extString = extString; }
+    const std::string& getExtensionString() { return _extString; }
+    void setExtensionString(const std::string& extString) { _extString = extString; }
     void eval(bool&value, const expression::Binding* b) const
     {
         int contextId = getOperand(0)->getValue(b);
         value = isGLExtensionSupported((unsigned)contextId, _extString.c_str());
     }
 protected:
-    string _extString;
+    std::string _extString;
 };
 
 Expression* extensionSupportedParser(const SGPropertyNode* exp,
