@@ -1,6 +1,6 @@
 // ReaderWriterSPT.cxx -- Provide a paged database for flightgear scenery.
 //
-// Copyright (C) 2010 - 2011  Mathias Froehlich
+// Copyright (C) 2010 - 2013  Mathias Froehlich
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as
@@ -27,9 +27,11 @@
 
 #include <osg/CullFace>
 #include <osg/PagedLOD>
+#include <osg/MatrixTransform>
 #include <osg/Texture2D>
 
 #include <osgDB/FileNameUtils>
+#include <osgDB/FileUtils>
 #include <osgDB/ReadFile>
 
 #include <simgear/scene/util/OsgMath.hxx>
@@ -109,6 +111,68 @@ struct ReaderWriterSPT::CullCallback : public osg::NodeCallback {
     }
 };
 
+struct ReaderWriterSPT::LocalOptions {
+    LocalOptions(const osgDB::Options* options) :
+        _options(options)
+    {
+        std::string pageLevelsString;
+        if (_options)
+            pageLevelsString = _options->getPluginStringData("SimGear::SPT_PAGE_LEVELS");
+
+        // Get the default if nothing given from outside
+        if (pageLevelsString.empty()) {
+            // We want an other level of indirection for paging
+            // Here we get about 12x12 deg tiles
+            _pageLevels.push_back(3);
+            // We want an other level of indirection for paging
+            // Here we get about 2x2 deg tiles
+            _pageLevels.push_back(5);
+            // We want an other level of indirection for paging
+            // Here we get about 0.5x0.5 deg tiles
+            _pageLevels.push_back(7);
+        } else {
+            // If configured from outside
+            std::stringstream ss(pageLevelsString);
+            while (ss.good()) {
+                unsigned level = ~0u;
+                ss >> level;
+                _pageLevels.push_back(level);
+            }
+        }
+    }
+
+    bool isPageLevel(unsigned level) const
+    {
+        return std::find(_pageLevels.begin(), _pageLevels.end(), level) != _pageLevels.end();
+    }
+
+    std::string getLodPathForBucketBox(const BucketBox& bucketBox) const
+    {
+        std::stringstream ss;
+        ss << "LOD/";
+        for (std::vector<unsigned>::const_iterator i = _pageLevels.begin(); i != _pageLevels.end(); ++i) {
+            if (bucketBox.getStartLevel() <= *i)
+                break;
+            ss << bucketBox.getParentBox(*i) << "/";
+        }
+        ss << bucketBox;
+        return ss.str();
+    }
+
+    float getRangeMultiplier() const
+    {
+        float rangeMultiplier = 2;
+        if (!_options)
+            return rangeMultiplier;
+        std::stringstream ss(_options->getPluginStringData("SimGear::SPT_RANGE_MULTIPLIER"));
+        ss >> rangeMultiplier;
+        return rangeMultiplier;
+    }
+
+    const osgDB::Options* _options;
+    std::vector<unsigned> _pageLevels;
+};
+
 ReaderWriterSPT::ReaderWriterSPT()
 {
     supportsExtension("spt", "SimGear paged terrain meta database.");
@@ -130,9 +194,9 @@ ReaderWriterSPT::readObject(const std::string& fileName, const osgDB::Options* o
     // We get called with different extensions. To make sure search continues,
     // we need to return FILE_NOT_HANDLED in this case.
     if (osgDB::getLowerCaseFileExtension(fileName) != "spt")
-        return ReadResult(osgDB::ReaderWriter::ReadResult::FILE_NOT_HANDLED);
+        return ReadResult(ReadResult::FILE_NOT_HANDLED);
     if (fileName != "state.spt")
-        return ReadResult(osgDB::ReaderWriter::ReadResult::FILE_NOT_FOUND);
+        return ReadResult(ReadResult::FILE_NOT_FOUND);
 
     osg::StateSet* stateSet = new osg::StateSet;
     stateSet->setAttributeAndModes(new osg::CullFace);
@@ -158,39 +222,42 @@ ReaderWriterSPT::readObject(const std::string& fileName, const osgDB::Options* o
 osgDB::ReaderWriter::ReadResult
 ReaderWriterSPT::readNode(const std::string& fileName, const osgDB::Options* options) const
 {
+    LocalOptions localOptions(options);
+
     // The file name without path and without the spt extension
     std::string strippedFileName = osgDB::getStrippedName(fileName);
     if (strippedFileName == "earth")
-        return createTree(BucketBox(0, -90, 360, 180), options, true);
+        return ReadResult(createTree(BucketBox(-180, -90, 360, 180), localOptions, true));
 
     std::stringstream ss(strippedFileName);
     BucketBox bucketBox;
     ss >> bucketBox;
     if (ss.fail())
-        return osgDB::ReaderWriter::ReadResult::FILE_NOT_FOUND;
+        return ReadResult::FILE_NOT_FOUND;
 
     BucketBox bucketBoxList[2];
     unsigned bucketBoxListSize = bucketBox.periodicSplit(bucketBoxList);
     if (bucketBoxListSize == 0)
-        return osgDB::ReaderWriter::ReadResult::FILE_NOT_FOUND;
+        return ReadResult::FILE_NOT_FOUND;
 
     if (bucketBoxListSize == 1)
-        return createTree(bucketBoxList[0], options, true);
+        return ReadResult(createTree(bucketBoxList[0], localOptions, true));
 
     assert(bucketBoxListSize == 2);
     osg::ref_ptr<osg::Group> group = new osg::Group;
-    group->addChild(createTree(bucketBoxList[0], options, true));
-    group->addChild(createTree(bucketBoxList[1], options, true));
-    return group.release();
+    group->addChild(createTree(bucketBoxList[0], localOptions, true));
+    group->addChild(createTree(bucketBoxList[1], localOptions, true));
+    return ReadResult(group);
 }
 
-osg::Node*
-ReaderWriterSPT::createTree(const BucketBox& bucketBox, const osgDB::Options* options, bool topLevel) const
+osg::ref_ptr<osg::Node>
+ReaderWriterSPT::createTree(const BucketBox& bucketBox, const LocalOptions& options, bool topLevel) const
 {
     if (bucketBox.getIsBucketSize()) {
-        return createPagedLOD(bucketBox, options);
-    } else if (!topLevel && bucketBox.getStartLevel() == 4) {
-        // We want an other level of indirection for paging
+        std::string fileName;
+        fileName = bucketBox.getBucket().gen_index_str() + std::string(".stg");
+        return osgDB::readRefNodeFile(fileName, options._options);
+    } else if (!topLevel && options.isPageLevel(bucketBox.getStartLevel())) {
         return createPagedLOD(bucketBox, options);
     } else {
         BucketBox bucketBoxList[100];
@@ -203,20 +270,20 @@ ReaderWriterSPT::createTree(const BucketBox& bucketBox, const osgDB::Options* op
 
         osg::ref_ptr<osg::Group> group = new osg::Group;
         for (unsigned i = 0; i < numTiles; ++i) {
-            osg::Node* node = createTree(bucketBoxList[i], options, false);
-            if (!node)
+            osg::ref_ptr<osg::Node> node = createTree(bucketBoxList[i], options, false);
+            if (!node.valid())
                 continue;
-            group->addChild(node);
+            group->addChild(node.get());
         }
         if (!group->getNumChildren())
             return 0;
 
-        return group.release();
+        return group;
     }
 }
 
-osg::Node*
-ReaderWriterSPT::createPagedLOD(const BucketBox& bucketBox, const osgDB::Options* options) const
+osg::ref_ptr<osg::Node>
+ReaderWriterSPT::createPagedLOD(const BucketBox& bucketBox, const LocalOptions& options) const
 {
     osg::PagedLOD* pagedLOD = new osg::PagedLOD;
 
@@ -228,45 +295,57 @@ ReaderWriterSPT::createPagedLOD(const BucketBox& bucketBox, const osgDB::Options
     pagedLOD->setCullCallback(new CullCallback);
 
     osg::ref_ptr<osgDB::Options> localOptions;
-    localOptions = static_cast<osgDB::Options*>(options->clone(osg::CopyOp()));
+    localOptions = static_cast<osgDB::Options*>(options._options->clone(osg::CopyOp()));
     // FIXME:
     // The particle systems have nodes with culling disabled.
     // PagedLOD nodes with childnodes like this will never expire.
     // So, for now switch them off.
     localOptions->setPluginStringData("SimGear::PARTICLESYSTEM", "OFF");
     pagedLOD->setDatabaseOptions(localOptions.get());
-        
-    float range;
-    if (bucketBox.getIsBucketSize())
-        range = 200e3;
-    else
-        range = 1e6;
 
-    // Add the static sea level textured shell
-    if (osg::Node* tile = createSeaLevelTile(bucketBox, options))
-        pagedLOD->addChild(tile, range, std::numeric_limits<float>::max());
+    // The break point for the low level of detail to the high level of detail
+    float rangeMultiplier = options.getRangeMultiplier();
+    float range = rangeMultiplier*sphere.getRadius();
+
+    // Look for a low level of detail tile
+    std::string lodPath = options.getLodPathForBucketBox(bucketBox);
+    const char* extensions[] = { ".btg.gz", ".flt" };
+    for (unsigned i = 0; i < sizeof(extensions)/sizeof(extensions[0]); ++i) {
+        std::string fileName = osgDB::findDataFile(lodPath + extensions[i], options._options);
+        if (fileName.empty())
+            continue;
+        osg::ref_ptr<osg::Node> node = osgDB::readRefNodeFile(fileName, options._options);
+        if (!node.valid())
+            continue;
+        pagedLOD->addChild(node.get(), range, std::numeric_limits<float>::max());
+        break;
+    }
+    // Add the static sea level textured shell if there is nothing found
+    if (pagedLOD->getNumChildren() == 0) {
+        osg::ref_ptr<osg::Node> node = createSeaLevelTile(bucketBox, options._options);
+        if (node.valid())
+            pagedLOD->addChild(node.get(), range, std::numeric_limits<float>::max());
+    }
 
     // Add the paged file name that creates the subtrees on demand
-    if (bucketBox.getIsBucketSize()) {
-        std::string fileName;
-        fileName = bucketBox.getBucket().gen_index_str() + std::string(".stg");
-        pagedLOD->setFileName(pagedLOD->getNumChildren(), fileName);
-    } else {
-        std::stringstream ss;
-        ss << bucketBox << ".spt";
-        pagedLOD->setFileName(pagedLOD->getNumChildren(), ss.str());
-    }
+    std::stringstream ss;
+    ss << bucketBox << ".spt";
+    pagedLOD->setFileName(pagedLOD->getNumChildren(), ss.str());
     pagedLOD->setRange(pagedLOD->getNumChildren(), 0.0, range);
 
     return pagedLOD;
 }
 
-osg::Node*
-ReaderWriterSPT::createSeaLevelTile(const BucketBox& bucketBox, const osgDB::Options* options) const
+osg::ref_ptr<osg::Node>
+ReaderWriterSPT::createSeaLevelTile(const BucketBox& bucketBox, const LocalOptions& options) const
 {
-    if (options->getPluginStringData("SimGear::FG_EARTH") != "ON")
+    if (options._options->getPluginStringData("SimGear::FG_EARTH") != "ON")
         return 0;
 
+    SGSpheref sphere = bucketBox.getBoundingSphere();
+    osg::Matrixd transform;
+    transform.makeTranslate(toOsg(-sphere.getCenter()));
+
     osg::Vec3Array* vertices = new osg::Vec3Array;
     osg::Vec3Array* normals = new osg::Vec3Array;
     osg::Vec2Array* texCoords = new osg::Vec2Array;
@@ -284,7 +363,7 @@ ReaderWriterSPT::createSeaLevelTile(const BucketBox& bucketBox, const osgDB::Opt
             SGVec2f t[6];
             unsigned num = bucketBox.getTileTriangles(i, j, incx, incy, v, n, t);
             for (unsigned k = 0; k < num; ++k) {
-                vertices->push_back(toOsg(v[k]));
+                vertices->push_back(transform.preMult(toOsg(v[k])));
                 normals->push_back(toOsg(n[k]));
                 texCoords->push_back(toOsg(t[k]));
             }
@@ -299,6 +378,8 @@ ReaderWriterSPT::createSeaLevelTile(const BucketBox& bucketBox, const osgDB::Opt
     colors->push_back(osg::Vec4(1, 1, 1, 1));
         
     osg::Geometry* geometry = new osg::Geometry;
+    geometry->setDataVariance(osg::Object::STATIC);
+    geometry->setUseVertexBufferObjects(true);
     geometry->setVertexArray(vertices);
     geometry->setNormalArray(normals);
     geometry->setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
@@ -306,27 +387,36 @@ ReaderWriterSPT::createSeaLevelTile(const BucketBox& bucketBox, const osgDB::Opt
     geometry->setColorBinding(osg::Geometry::BIND_OVERALL);
     geometry->setTexCoordArray(0, texCoords);
         
-    geometry->addPrimitiveSet(new osg::DrawArrays(osg::DrawArrays::TRIANGLES, 0, vertices->size()));
+    osg::DrawArrays* drawArrays = new osg::DrawArrays(osg::DrawArrays::TRIANGLES, 0, vertices->size());
+    drawArrays->setDataVariance(osg::Object::STATIC);
+    geometry->addPrimitiveSet(drawArrays);
         
     osg::Geode* geode = new osg::Geode;
+    geode->setDataVariance(osg::Object::STATIC);
     geode->addDrawable(geometry);
-    geode->setStateSet(getLowLODStateSet(options));
+    osg::ref_ptr<osg::StateSet> stateSet = getLowLODStateSet(options);
+    geode->setStateSet(stateSet.get());
 
-    return geode;
+    transform.makeTranslate(toOsg(sphere.getCenter()));
+    osg::MatrixTransform* matrixTransform = new osg::MatrixTransform(transform);
+    matrixTransform->setDataVariance(osg::Object::STATIC);
+    matrixTransform->addChild(geode);
+
+    return matrixTransform;
 }
 
-osg::StateSet*
-ReaderWriterSPT::getLowLODStateSet(const osgDB::Options* options) const
+osg::ref_ptr<osg::StateSet>
+ReaderWriterSPT::getLowLODStateSet(const LocalOptions& options) const
 {
     osg::ref_ptr<osgDB::Options> localOptions;
-    localOptions = static_cast<osgDB::Options*>(options->clone(osg::CopyOp()));
+    localOptions = static_cast<osgDB::Options*>(options._options->clone(osg::CopyOp()));
     localOptions->setObjectCacheHint(osgDB::Options::CACHE_ALL);
 
-    osg::ref_ptr<osg::Object> object = osgDB::readObjectFile("state.spt", localOptions.get());
+    osg::ref_ptr<osg::Object> object = osgDB::readRefObjectFile("state.spt", localOptions.get());
     if (!dynamic_cast<osg::StateSet*>(object.get()))
         return 0;
 
-    return static_cast<osg::StateSet*>(object.release());
+    return static_cast<osg::StateSet*>(object.get());
 }
 
 } // namespace simgear
