@@ -27,6 +27,7 @@
 
 #include <osg/LOD>
 #include <osg/MatrixTransform>
+#include <osg/PagedLOD>
 #include <osg/ProxyNode>
 #include <osgUtil/LineSegmentIntersector>
 #include <osgUtil/IntersectionVisitor>
@@ -37,12 +38,14 @@
 #include <osgDB/ReaderWriter>
 #include <osgDB/ReadFile>
 
+#include <simgear/math/SGGeometry.hxx>
 #include <simgear/bucket/newbucket.hxx>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sgstream.hxx>
-#include <simgear/scene/util/SGReaderWriterOptions.hxx>
-#include <simgear/scene/util/RenderConstants.hxx>
+#include <simgear/scene/util/OptionsReadFileCallback.hxx>
 #include <simgear/scene/util/OsgMath.hxx>
+#include <simgear/scene/util/RenderConstants.hxx>
+#include <simgear/scene/util/SGReaderWriterOptions.hxx>
 #include <simgear/scene/tgdb/apt_signs.hxx>
 #include <simgear/scene/tgdb/obj.hxx>
 
@@ -109,6 +112,66 @@ struct ReaderWriterSTG::_ModelBin {
         int _size;
     };
 
+    class DelayLoadReadFileCallback : public OptionsReadFileCallback {
+    public:
+        virtual osgDB::ReaderWriter::ReadResult
+        readNode(const std::string&, const osgDB::Options*)
+        {
+            osg::ref_ptr<osg::Group> group = new osg::Group;
+            group->setName("STG-group-A");
+            group->setDataVariance(osg::Object::STATIC);
+            
+            for (std::list<_ObjectStatic>::iterator i = _objectStaticList.begin(); i != _objectStaticList.end(); ++i) {
+                osg::ref_ptr<osg::Node> node;
+                if (i->_proxy)  {
+                    osg::ref_ptr<osg::ProxyNode> proxy = new osg::ProxyNode;
+                    proxy->setName("proxyNode");
+                    proxy->setLoadingExternalReferenceMode(osg::ProxyNode::DEFER_LOADING_TO_DATABASE_PAGER);
+                    proxy->setFileName(0, i->_name);
+                    proxy->setDatabaseOptions(i->_options.get());
+                    node = proxy;
+                } else {
+                    node = osgDB::readRefNodeFile(i->_name, i->_options.get());
+                    if (!node.valid()) {
+                        SG_LOG(SG_TERRAIN, SG_ALERT, i->_errorLocation << ": Failed to load "
+                               << i->_token << " '" << i->_name << "'");
+                        continue;
+                    }
+                }
+                if (SGPath(i->_name).lower_extension() == "ac")
+                    node->setNodeMask(~simgear::MODELLIGHT_BIT);
+                
+                osg::Matrix matrix;
+                matrix = makeZUpFrame(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev));
+                matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_hdg), osg::Vec3(0, 0, 1)));
+                matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_pitch), osg::Vec3(0, 1, 0)));
+                matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_roll), osg::Vec3(1, 0, 0)));
+                
+                osg::MatrixTransform* matrixTransform;
+                matrixTransform = new osg::MatrixTransform(matrix);
+                matrixTransform->setName("positionStaticObject");
+                matrixTransform->setDataVariance(osg::Object::STATIC);
+                matrixTransform->addChild(node.get());
+                group->addChild(matrixTransform);
+            }
+            
+            simgear::AirportSignBuilder signBuilder(_options->getMaterialLib(), _bucket.get_center());
+            for (std::list<_Sign>::iterator i = _signList.begin(); i != _signList.end(); ++i)
+                signBuilder.addSign(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev), i->_hdg, i->_name, i->_size);
+            if (signBuilder.getSignsGroup())
+                group->addChild(signBuilder.getSignsGroup());
+            
+            return group.release();
+        }
+        
+        std::list<_ObjectStatic> _objectStaticList;
+        std::list<_Sign> _signList;
+        
+        /// The original options to use for this bunch of models
+        osg::ref_ptr<SGReaderWriterOptions> _options;
+        SGBucket _bucket;
+    };
+    
     _ModelBin() :
         _foundBase(false)
     { }
@@ -122,9 +185,25 @@ struct ReaderWriterSTG::_ModelBin {
         SGPath path = filePath;
         path.append(".."); path.append(".."); path.append("..");
         sharedOptions->getDatabasePathList().push_back(path.str());
+        
+        // ensure Models directory synced via TerraSync is searched before the copy in
+        // FG_ROOT, so that updated models can be used.
+        std::string terrasync_root = options->getPluginStringData("SimGear::TERRASYNC_ROOT");
+        if (!terrasync_root.empty()) {
+            sharedOptions->getDatabasePathList().push_back(terrasync_root);
+        }
+        
         std::string fg_root = options->getPluginStringData("SimGear::FG_ROOT");
         sharedOptions->getDatabasePathList().push_back(fg_root);
 
+        // TODO how should we handle this for OBJECT_SHARED?
+        sharedOptions->setModelData
+        (
+            sharedOptions->getModelData()
+          ? sharedOptions->getModelData()->clone()
+          : 0
+        );
+
         return sharedOptions.release();
     }
     SGReaderWriterOptions* staticOptions(const std::string& filePath, const osgDB::Options* options)
@@ -136,6 +215,15 @@ struct ReaderWriterSTG::_ModelBin {
         staticOptions->getDatabasePathList().push_back(filePath);
         staticOptions->setObjectCacheHint(osgDB::Options::CACHE_NONE);
 
+        // Every model needs its own SGModelData to ensure load/unload is
+        // working properly
+        staticOptions->setModelData
+        (
+            staticOptions->getModelData()
+          ? staticOptions->getModelData()->clone()
+          : 0
+        );
+
         return staticOptions.release();
     }
 
@@ -291,7 +379,8 @@ struct ReaderWriterSTG::_ModelBin {
 
         osg::ref_ptr<osg::Group> terrainGroup = new osg::Group;
         terrainGroup->setDataVariance(osg::Object::STATIC);
-
+        terrainGroup->setName("terrain");
+        
         if (_foundBase) {
             for (std::list<_Object>::iterator i = _objectList.begin(); i != _objectList.end(); ++i) {
                 osg::ref_ptr<osg::Node> node;
@@ -304,10 +393,11 @@ struct ReaderWriterSTG::_ModelBin {
                 terrainGroup->addChild(node.get());
             }
         } else {
-            SG_LOG(SG_TERRAIN, SG_INFO, "  Generating ocean tile");
+            SG_LOG(SG_TERRAIN, SG_INFO, "  Generating ocean tile: " << bucket.gen_base_path() << "/" << bucket.gen_index_str());
             
             osg::Node* node = SGOceanTile(bucket, options->getMaterialLib());
             if (node) {
+                node->setName("SGOceanTile");
                 terrainGroup->addChild(node);
             } else {
                 SG_LOG( SG_TERRAIN, SG_ALERT,
@@ -326,58 +416,36 @@ struct ReaderWriterSTG::_ModelBin {
                 continue;
             i->_elev += elevation(*terrainGroup, SGGeod::fromDeg(i->_lon, i->_lat));
         }
-        
-        osg::ref_ptr<osg::Group> modelGroup = new osg::Group;
-        modelGroup->setDataVariance(osg::Object::STATIC);
 
-        for (std::list<_ObjectStatic>::iterator i = _objectStaticList.begin(); i != _objectStaticList.end(); ++i) {
-            osg::ref_ptr<osg::Node> node;
-            if (i->_proxy)  {
-                osg::ref_ptr<osg::ProxyNode> proxy = new osg::ProxyNode;
-                proxy->setLoadingExternalReferenceMode(osg::ProxyNode::DEFER_LOADING_TO_DATABASE_PAGER);
-                proxy->setFileName(0, i->_name);
-                proxy->setDatabaseOptions(i->_options.get());
-                node = proxy;
-            } else {
-                node = osgDB::readRefNodeFile(i->_name, i->_options.get());
-                if (!node.valid()) {
-                    SG_LOG(SG_TERRAIN, SG_ALERT, i->_errorLocation << ": Failed to load "
-                           << i->_token << " '" << i->_name << "'");
-                    continue;
-                }
-            }
-            if (SGPath(i->_name).lower_extension() == "ac")
-                node->setNodeMask(~simgear::MODELLIGHT_BIT);
+        if (_objectStaticList.empty() && _signList.empty()) {
+            // The simple case, just return the terrain group
+            return terrainGroup.release();
+        } else {
+            osg::PagedLOD* pagedLOD = new osg::PagedLOD;
+            pagedLOD->setCenterMode(osg::PagedLOD::USE_BOUNDING_SPHERE_CENTER);
+            pagedLOD->setName("pagedObjectLOD");
             
-            osg::Matrix matrix;
-            matrix = makeZUpFrame(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev));
-            matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_hdg), osg::Vec3(0, 0, 1)));
-            matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_pitch), osg::Vec3(0, 1, 0)));
-            matrix.preMultRotate(osg::Quat(SGMiscd::deg2rad(i->_roll), osg::Vec3(1, 0, 0)));
+            // This should be visible in any case.
+            // If this is replaced by some lower level of detail, the parent LOD node handles this.
+            pagedLOD->addChild(terrainGroup, 0, std::numeric_limits<float>::max());
+
+            // we just need to know about the read file callback that itself holds the data
+            osg::ref_ptr<DelayLoadReadFileCallback> readFileCallback = new DelayLoadReadFileCallback;
+            readFileCallback->_objectStaticList = _objectStaticList;
+            readFileCallback->_signList = _signList;
+            readFileCallback->_options = options;
+            readFileCallback->_bucket = bucket;
+            osg::ref_ptr<osgDB::Options> callbackOptions = new osgDB::Options;
+            callbackOptions->setReadFileCallback(readFileCallback.get());
+            pagedLOD->setDatabaseOptions(callbackOptions.get());
             
-            osg::MatrixTransform* matrixTransform;
-            matrixTransform = new osg::MatrixTransform(matrix);
-            matrixTransform->setDataVariance(osg::Object::STATIC);
-            matrixTransform->addChild(node.get());
-            modelGroup->addChild(matrixTransform);
+            pagedLOD->setFileName(pagedLOD->getNumChildren(), "Dummy name - use the stored data in the read file callback");
+            pagedLOD->setRange(pagedLOD->getNumChildren(), 0, 30000);
+            
+            return pagedLOD;
         }
-        
-        simgear::AirportSignBuilder signBuilder(options->getMaterialLib(), bucket.get_center());
-        for (std::list<_Sign>::iterator i = _signList.begin(); i != _signList.end(); ++i)
-            signBuilder.addSign(SGGeod::fromDegM(i->_lon, i->_lat, i->_elev), i->_hdg, i->_name, i->_size);
-        if (signBuilder.getSignsGroup())
-            modelGroup->addChild(signBuilder.getSignsGroup());
-
-        osg::LOD* lod = new osg::LOD;
-        lod->setDataVariance(osg::Object::STATIC);
-        if (terrainGroup->getNumChildren())
-            lod->addChild(terrainGroup.get(), 0, std::numeric_limits<float>::max());
-        if (modelGroup->getNumChildren())
-            lod->addChild(modelGroup.get(), 0, 30000);
-        
-        return lod;
     }
-        
+    
     bool _foundBase;
     std::list<_Object> _objectList;
     std::list<_ObjectStatic> _objectStaticList;
