@@ -33,14 +33,12 @@
 #endif
 
 #ifdef __MINGW32__
-#include <time.h>
-#include <unistd.h>
+#  include <time.h>
+#  include <unistd.h>
 #elif defined(_MSC_VER)
 #   include <io.h>
-#   ifndef HAVE_SVN_CLIENT_H
-#       include <time.h>
-#       include <process.h>
-#   endif
+#   include <time.h>
+#   include <process.h>
 #endif
 
 #include <stdlib.h>             // atoi() atof() abs() system()
@@ -55,34 +53,19 @@
 #include <simgear/compiler.h>
 
 #include "terrasync.hxx"
+
 #include <simgear/bucket/newbucket.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/threads/SGQueue.hxx>
 #include <simgear/misc/sg_dir.hxx>
+#include <simgear/debug/BufferedLogCallback.hxx>
+#include <simgear/props/props_io.hxx>
+#include <simgear/io/HTTPClient.hxx>
+#include <simgear/io/SVNRepository.hxx>
+#include <simgear/structure/exception.hxx>
 
-#ifdef HAVE_SVN_CLIENT_H
-#  ifdef HAVE_LIBSVN_CLIENT_1
-#    include <svn_version.h>
-#    include <svn_auth.h>
-#    include <svn_client.h>
-#    include <svn_cmdline.h>
-#    include <svn_pools.h>
-#  else
-#    undef HAVE_SVN_CLIENT_H
-#  endif
-#endif
-
-#ifdef HAVE_SVN_CLIENT_H
-    static const svn_version_checklist_t mysvn_checklist[] = {
-        { "svn_subr",   svn_subr_version },
-        { "svn_client", svn_client_version },
-        { NULL, NULL }
-    };
-    static const bool svn_built_in_available = true;
-#else
-    static const bool svn_built_in_available = false;
-#endif
+static const bool svn_built_in_available = true;
 
 using namespace simgear;
 using namespace std;
@@ -125,17 +108,98 @@ bool hasWhitespace(string path)
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// WaitingTile ////////////////////////////////////////////////////////////////
+// SyncItem ////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
-class  WaitingTile
+class SyncItem
 {
 public:
-    WaitingTile(string dir,bool refresh) :
-        _dir(dir), _refreshScenery(refresh) {}
+    enum Type
+    {
+        Stop = 0,   ///< special item indicating to stop the SVNThread
+        Tile,
+        AirportData,
+        SharedModels,
+        AIData
+    };
+    
+    enum Status
+    {
+        Invalid = 0,
+        Waiting,
+        Cached, ///< using already cached result
+        Updated,
+        NotFound, 
+        Failed
+    };
+    
+    SyncItem() :
+        _dir(),
+        _type(Stop),
+        _status(Invalid)
+    {
+    }
+    
+    SyncItem(string dir, Type ty) :
+        _dir(dir),
+        _type(ty),
+        _status(Waiting)
+    {}
+        
     string _dir;
-    bool _refreshScenery;
+    Type _type;
+    Status _status;
 };
 
+///////////////////////////////////////////////////////////////////////////////
+
+/**
+ * @brief SyncSlot encapsulates a queue of sync items we will fetch
+ * serially. Multiple slots exist to sync different types of item in
+ * parallel.
+ */
+class SyncSlot
+{
+public:
+    SyncSlot() :
+        isNewDirectory(false),
+        busy(false)
+    {}
+    
+    SyncItem currentItem;
+    bool isNewDirectory;
+    std::queue<SyncItem> queue;
+    std::auto_ptr<SVNRepository> repository;
+    SGTimeStamp stamp;
+    bool busy; ///< is the slot working or idle
+    
+    unsigned int nextWarnTimeout;
+};
+
+static const int SYNC_SLOT_TILES = 0; ///< Terrain and Objects sync
+static const int SYNC_SLOT_SHARED_DATA = 1; /// shared Models and Airport data
+static const int SYNC_SLOT_AI_DATA = 2; /// AI traffic and models
+static const unsigned int NUM_SYNC_SLOTS = 3;
+
+/**
+ * @brief translate a sync item type into one of the available slots.
+ * This provides the scheduling / balancing / prioritising between slots.
+ */
+static unsigned int syncSlotForType(SyncItem::Type ty)
+{
+    switch (ty) {
+    case SyncItem::Tile: return SYNC_SLOT_TILES;
+    case SyncItem::SharedModels:
+    case SyncItem::AirportData:
+        return SYNC_SLOT_SHARED_DATA;
+    case SyncItem::AIData:
+        return SYNC_SLOT_AI_DATA;
+        
+    default:
+        return SYNC_SLOT_SHARED_DATA;
+    }
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 // SGTerraSync::SvnThread /////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
@@ -148,13 +212,15 @@ public:
    void stop();
    bool start();
 
-   bool isIdle() {return waitingTiles.empty();}
-   void request(const WaitingTile& dir) {waitingTiles.push_front(dir);}
+    bool isIdle() {return !_busy; }
+   void request(const SyncItem& dir) {waitingTiles.push_front(dir);}
    bool isDirty() { bool r = _is_dirty;_is_dirty = false;return r;}
    bool hasNewTiles() { return !_freshTiles.empty();}
-   WaitingTile getNewTile() { return _freshTiles.pop_front();}
+   SyncItem getNewTile() { return _freshTiles.pop_front();}
 
    void   setSvnServer(string server)       { _svn_server   = stripPath(server);}
+   void   setSvnDataServer(string server)   { _svn_data_server   = stripPath(server);}
+    
    void   setExtSvnUtility(string svn_util) { _svn_command  = simgear::strutils::strip(svn_util);}
    void   setRsyncServer(string server)     { _rsync_server = simgear::strutils::strip(server);}
    void   setLocalDir(string dir)           { _local_dir    = stripPath(dir);}
@@ -162,9 +228,9 @@ public:
    void   setUseSvn(bool use_svn)           { _use_svn = use_svn;}
    void   setAllowedErrorCount(int errors)  {_allowed_errors = errors;}
 
-#ifdef HAVE_SVN_CLIENT_H
-   void setUseBuiltin(bool built_in) { _use_built_in = built_in;}
-#endif
+   void   setCachePath(const SGPath& p)     {_persistentCachePath = p;}
+   void   setCacheHits(unsigned int hits)   {_cache_hits = hits;}
+   void   setUseBuiltin(bool built_in) { _use_built_in = built_in;}
 
    volatile bool _active;
    volatile bool _running;
@@ -175,44 +241,49 @@ public:
    volatile int  _success_count;
    volatile int  _consecutive_errors;
    volatile int  _allowed_errors;
-
+   volatile int  _cache_hits;
+   volatile int _transfer_rate;
+   // kbytes, not bytes, because bytes might overflow 2^31
+   volatile int _total_kb_downloaded;
+   
 private:
    virtual void run();
-   bool syncTree(const char* dir, bool& isNewDirectory);
-   bool syncTreeExternal(const char* dir);
+    
+    // external model run and helpers
+    void runExternal();
+    void syncPathExternal(const SyncItem& next);
+    bool runExternalSyncCommand(const char* dir);
 
-#ifdef HAVE_SVN_CLIENT_H
-   static int svnClientSetup(void);
-   bool syncTreeInternal(const char* dir);
+    // internal mode run and helpers
+    void runInternal();
+    void updateSyncSlot(SyncSlot& slot);
 
-   bool _use_built_in;
-
-   // Things we need for doing subversion checkout - often
-   static apr_pool_t *_svn_pool;
-   static svn_client_ctx_t *_svn_ctx;
-   static svn_opt_revision_t *_svn_rev;
-   static svn_opt_revision_t *_svn_rev_peg;
-#endif
+    // commond helpers between both internal and external models
+    
+    bool isPathCached(const SyncItem& next) const;
+    void initCompletedTilesPersistentCache();
+    void writeCompletedTilesPersistentCache() const;
+    void updated(SyncItem item, bool isNewDirectory);
+    void fail(SyncItem failedItem);
+    
+    bool _use_built_in;
+    HTTP::Client _http;
+    SyncSlot _syncSlots[NUM_SYNC_SLOTS];
 
    volatile bool _is_dirty;
    volatile bool _stop;
-   SGBlockingDeque <WaitingTile> waitingTiles;
+   SGBlockingDeque <SyncItem> waitingTiles;
    CompletedTiles _completedTiles;
-   SGBlockingDeque <WaitingTile> _freshTiles;
+   SGBlockingDeque <SyncItem> _freshTiles;
    bool _use_svn;
    string _svn_server;
+   string _svn_data_server;
    string _svn_command;
    string _rsync_server;
    string _local_dir;
+   SGPath _persistentCachePath;
 };
 
-#ifdef HAVE_SVN_CLIENT_H
-    apr_pool_t* SGTerraSync::SvnThread::_svn_pool = NULL;
-    svn_client_ctx_t* SGTerraSync::SvnThread::_svn_ctx = NULL;
-    svn_opt_revision_t* SGTerraSync::SvnThread::_svn_rev = NULL;
-    svn_opt_revision_t* SGTerraSync::SvnThread::_svn_rev_peg = NULL;
-#endif
-
 SGTerraSync::SvnThread::SvnThread() :
     _active(false),
     _running(false),
@@ -223,21 +294,15 @@ SGTerraSync::SvnThread::SvnThread() :
     _success_count(0),
     _consecutive_errors(0),
     _allowed_errors(6),
-#ifdef HAVE_SVN_CLIENT_H
+    _cache_hits(0),
+    _transfer_rate(0),
+    _total_kb_downloaded(0),
     _use_built_in(true),
-#endif
     _is_dirty(false),
     _stop(false),
     _use_svn(true)
 {
-#ifdef HAVE_SVN_CLIENT_H
-    int errCode = SGTerraSync::SvnThread::svnClientSetup();
-    if (errCode != EXIT_SUCCESS)
-    {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
-               "Failed to initialize built-in SVN client, error = " << errCode);
-    }
-#endif
+    _http.setUserAgent("terrascenery-" SG_STRINGIZE(SG_VERSION));
 }
 
 void SGTerraSync::SvnThread::stop()
@@ -250,7 +315,7 @@ void SGTerraSync::SvnThread::stop()
 
     // set stop flag and wake up the thread with an empty request
     _stop = true;
-    WaitingTile w("",false);
+    SyncItem w(string(), SyncItem::Stop);
     request(w);
     join();
     _running = false;
@@ -292,9 +357,7 @@ bool SGTerraSync::SvnThread::start()
         return false;
     }
 
-#ifdef HAVE_SVN_CLIENT_H
     _use_svn |= _use_built_in;
-#endif
 
     if ((_use_svn)&&(_svn_server==""))
     {
@@ -322,12 +385,10 @@ bool SGTerraSync::SvnThread::start()
     _running = true;
 
     string status;
-#ifdef HAVE_SVN_CLIENT_H
+
     if (_use_svn && _use_built_in)
         status = "Using built-in SVN support. ";
-    else
-#endif
-    if (_use_svn)
+    else if (_use_svn)
     {
         status = "Using external SVN utility '";
         status += _svn_command;
@@ -349,114 +410,7 @@ bool SGTerraSync::SvnThread::start()
     return true;
 }
 
-// sync one directory tree
-bool SGTerraSync::SvnThread::syncTree(const char* dir, bool& isNewDirectory)
-{
-    int rc;
-    SGPath path( _local_dir );
-
-    path.append( dir );
-    isNewDirectory = !path.exists();
-    if (isNewDirectory)
-    {
-        rc = path.create_dir( 0755 );
-        if (rc)
-        {
-            SG_LOG(SG_TERRAIN,SG_ALERT,
-                   "Cannot create directory '" << dir << "', return code = " << rc );
-            return false;
-        }
-    }
-
-#ifdef HAVE_SVN_CLIENT_H
-    if (_use_built_in)
-        return syncTreeInternal(dir);
-    else
-#endif
-    {
-        return syncTreeExternal(dir);
-    }
-}
-
-
-#ifdef HAVE_SVN_CLIENT_H
-bool SGTerraSync::SvnThread::syncTreeInternal(const char* dir)
-{
-    SG_LOG(SG_TERRAIN,SG_DEBUG, "Synchronizing scenery directory " << dir);
-    if (!_svn_pool)
-    {
-        SG_LOG(SG_TERRAIN,SG_ALERT,
-               "Built-in SVN client failed to initialize.");
-        return false;
-    }
-
-    ostringstream command;
-    command << _svn_server << "/" << dir;
-
-    ostringstream dest_base_dir;
-    dest_base_dir << _local_dir << "/" << dir;
-
-    apr_pool_t *subpool = svn_pool_create(_svn_pool);
-
-    svn_error_t *err = NULL;
-#if (SVN_VER_MINOR >= 5)
-    err = svn_client_checkout3(NULL,
-            command.str().c_str(),
-            dest_base_dir.str().c_str(),
-            _svn_rev_peg,
-            _svn_rev,
-            svn_depth_infinity,
-            0, // ignore-externals = false
-            0, // allow unver obstructions = false
-            _svn_ctx,
-            subpool);
-#else
-    // version 1.4 API
-    err = svn_client_checkout2(NULL,
-            command.str().c_str(),
-            dest_base_dir.str().c_str(),
-            _svn_rev_peg,
-            _svn_rev,
-            1, // recurse=true - same as svn_depth_infinity for checkout3 above
-            0, // ignore externals = false
-            _svn_ctx,
-            subpool);
-#endif
-
-    bool ReturnValue = true;
-    if (err)
-    {
-        // Report errors from the checkout attempt
-        if (err->apr_err == SVN_ERR_RA_ILLEGAL_URL)
-        {
-            // ignore errors when remote path doesn't exist (no scenery data for ocean areas)
-        }
-        else
-        {
-            SG_LOG(SG_TERRAIN,SG_ALERT,
-                    "Failed to synchronize directory '" << dir << "', " <<
-                    err->message << " (code " << err->apr_err << ").");
-            svn_error_clear(err);
-            // try to clean up
-            err = svn_client_cleanup(dest_base_dir.str().c_str(),
-                    _svn_ctx,subpool);
-            if (!err)
-            {
-                SG_LOG(SG_TERRAIN,SG_ALERT,
-                       "SVN repository cleanup successful for '" << dir << "'.");
-            }
-            ReturnValue = false;
-        }
-    } else
-    {
-        SG_LOG(SG_TERRAIN,SG_DEBUG, "Done with scenery directory " << dir);
-    }
-    svn_pool_destroy(subpool);
-    return ReturnValue;
-}
-#endif
-
-bool SGTerraSync::SvnThread::syncTreeExternal(const char* dir)
+bool SGTerraSync::SvnThread::runExternalSyncCommand(const char* dir)
 {
     ostringstream buf;
     SGPath localPath( _local_dir );
@@ -513,49 +467,39 @@ bool SGTerraSync::SvnThread::syncTreeExternal(const char* dir)
 void SGTerraSync::SvnThread::run()
 {
     _active = true;
+    initCompletedTilesPersistentCache();
+
+    
+    if (_use_built_in) {
+        runInternal();
+    } else {
+        runExternal();
+    }
+    
+    _active = false;
+    _running = false;
+    _is_dirty = true;
+}
+
+void SGTerraSync::SvnThread::runExternal()
+{
     while (!_stop)
     {
-        WaitingTile next = waitingTiles.pop_front();
+        SyncItem next = waitingTiles.pop_front();
         if (_stop)
            break;
 
-        CompletedTiles::iterator ii =
-            _completedTiles.find( next._dir );
-        time_t now = time(0);
-        if ((ii == _completedTiles.end())||
-            (ii->second < now ))
-        {
-            bool isNewDirectory = false;
-
-            _busy = true;
-            if (!syncTree(next._dir.c_str(),isNewDirectory))
-            {
-                _consecutive_errors++;
-                _fail_count++;
-                _completedTiles[ next._dir ] = now + UpdateInterval::FailedAttempt;
-            }
-            else
-            {
-                _consecutive_errors = 0;
-                _success_count++;
-                SG_LOG(SG_TERRAIN,SG_INFO,
-                       "Successfully synchronized directory '" << next._dir << "'");
-                if (next._refreshScenery)
-                {
-                    // updated a tile
-                    _updated_tile_count++;
-                    if (isNewDirectory)
-                    {
-                        // for now only report new directories to refresh display
-                        // (i.e. only when ocean needs to be replaced with actual data)
-                        _freshTiles.push_back(next);
-                        _is_dirty = true;
-                    }
-                }
-                _completedTiles[ next._dir ] = now + UpdateInterval::SuccessfulAttempt;
-            }
-            _busy = false;
+        if (isPathCached(next)) {
+            _cache_hits++;
+            SG_LOG(SG_TERRAIN, SG_DEBUG,
+                   "Cache hit for: '" << next._dir << "'");
+            next._status = SyncItem::Cached;
+            _freshTiles.push_back(next);
+            _is_dirty = true;
+            continue;
         }
+        
+        syncPathExternal(next);
 
         if ((_allowed_errors >= 0)&&
             (_consecutive_errors >= _allowed_errors))
@@ -563,163 +507,323 @@ void SGTerraSync::SvnThread::run()
             _stalled = true;
             _stop = true;
         }
-    }
+    } // of thread running loop
+}
 
-    _active = false;
-    _running = false;
+void SGTerraSync::SvnThread::syncPathExternal(const SyncItem& next)
+{
+    _busy = true;
+    SGPath path( _local_dir );
+    path.append( next._dir );
+    bool isNewDirectory = !path.exists();
+    
+    try {
+        if (isNewDirectory) {
+            int rc = path.create_dir( 0755 );
+            if (rc) {
+                SG_LOG(SG_TERRAIN,SG_ALERT,
+                       "Cannot create directory '" << path << "', return code = " << rc );
+                throw sg_exception("Cannot create directory for terrasync", path.str());
+            }
+        }
+        
+        if (!runExternalSyncCommand(next._dir.c_str())) {
+            throw sg_exception("Running external sync command failed");
+        }
+    } catch (sg_exception& e) {
+        fail(next);
+        _busy = false;
+        return;
+    }
+    
+    updated(next, isNewDirectory);
+    _busy = false;
+}
+
+void SGTerraSync::SvnThread::updateSyncSlot(SyncSlot &slot)
+{
+    if (slot.repository.get()) {
+        if (slot.repository->isDoingSync()) {
+#if 0
+            if (slot.stamp.elapsedMSec() > slot.nextWarnTimeout) {
+                SG_LOG(SG_TERRAIN, SG_INFO, "sync taking a long time:" << slot.currentItem._dir << " taken " << slot.stamp.elapsedMSec());
+                SG_LOG(SG_TERRAIN, SG_INFO, "HTTP status:" << _http.hasActiveRequests());
+                slot.nextWarnTimeout += 10000;
+            }
+#endif
+            return; // easy, still working
+        }
+        
+        // check result
+        SVNRepository::ResultCode res = slot.repository->failure();
+        if (res == SVNRepository::SVN_ERROR_NOT_FOUND) {
+            // this is fine, but maybe we should use a different return code
+            // in the future to higher layers can distinguish this case
+            slot.currentItem._status = SyncItem::NotFound;
+            _freshTiles.push_back(slot.currentItem);
+            _is_dirty = true;
+        } else if (res != SVNRepository::SVN_NO_ERROR) {
+            fail(slot.currentItem);
+        } else {
+            updated(slot.currentItem, slot.isNewDirectory);
+            SG_LOG(SG_TERRAIN, SG_DEBUG, "sync of " << slot.repository->baseUrl() << " finished ("
+                   << slot.stamp.elapsedMSec() << " msec");
+        }
+
+        // whatever happened, we're done with this repository instance
+        slot.busy = false;
+        slot.repository.reset();
+    }
+    
+    // init and start sync of the next repository
+    if (!slot.queue.empty()) {
+        slot.currentItem = slot.queue.front();
+        slot.queue.pop();
+        
+        SGPath path(_local_dir);
+        path.append(slot.currentItem._dir);
+        slot.isNewDirectory = !path.exists();
+        if (slot.isNewDirectory) {
+            int rc = path.create_dir( 0755 );
+            if (rc) {
+                SG_LOG(SG_TERRAIN,SG_ALERT,
+                       "Cannot create directory '" << path << "', return code = " << rc );
+                fail(slot.currentItem);
+                return;
+            }
+        } // of creating directory step
+        
+        string serverUrl(_svn_server);
+        if (slot.currentItem._type == SyncItem::AIData) {
+            serverUrl = _svn_data_server;
+        }
+        
+        slot.repository.reset(new SVNRepository(path, &_http));
+        slot.repository->setBaseUrl(serverUrl + "/" + slot.currentItem._dir);
+        slot.repository->update();
+        
+        slot.nextWarnTimeout = 20000;
+        slot.stamp.stamp();
+        slot.busy = true;
+        SG_LOG(SG_TERRAIN, SG_DEBUG, "sync of " << slot.repository->baseUrl() << " started, queue size is " << slot.queue.size());
+    }
+}
+
+void SGTerraSync::SvnThread::runInternal()
+{
+    while (!_stop) {
+        _http.update(100);
+        _transfer_rate = _http.transferRateBytesPerSec();
+        // convert from bytes to kbytes
+        _total_kb_downloaded = static_cast<int>(_http.totalBytesDownloaded() / 1024);
+        
+        if (_stop)
+            break;
+ 
+        // drain the waiting tiles queue into the sync slot queues.
+        while (!waitingTiles.empty()) {
+            SyncItem next = waitingTiles.pop_front();
+            if (isPathCached(next)) {
+                _cache_hits++;
+                SG_LOG(SG_TERRAIN, SG_DEBUG, "TerraSync Cache hit for: '" << next._dir << "'");
+                next._status = SyncItem::Cached;
+                _freshTiles.push_back(next);
+                _is_dirty = true;
+                continue;
+            }
+
+            unsigned int slot = syncSlotForType(next._type);
+            _syncSlots[slot].queue.push(next);
+        }
+       
+        bool anySlotBusy = false;
+        // update each sync slot in turn
+        for (unsigned int slot=0; slot < NUM_SYNC_SLOTS; ++slot) {
+            updateSyncSlot(_syncSlots[slot]);
+            anySlotBusy |= _syncSlots[slot].busy;
+        }
+
+        _busy = anySlotBusy;
+        if (!anySlotBusy) {
+            // wait on the blocking deque here, otherwise we spin
+            // the loop very fast, since _http::update with no connections
+            // active returns immediately.
+            waitingTiles.waitOnNotEmpty();
+        }
+    } // of thread running loop
+}
+
+bool SGTerraSync::SvnThread::isPathCached(const SyncItem& next) const
+{
+    CompletedTiles::const_iterator ii = _completedTiles.find( next._dir );
+    if (ii == _completedTiles.end()) {
+        return false;
+    }
+    
+    // check if the path still physically exists
+    SGPath p(_local_dir);
+    p.append(next._dir);
+    if (!p.exists()) {
+        return false;
+    }
+    
+    time_t now = time(0);
+    return (ii->second > now);
+}
+
+void SGTerraSync::SvnThread::fail(SyncItem failedItem)
+{
+    time_t now = time(0);
+    _consecutive_errors++;
+    _fail_count++;
+    failedItem._status = SyncItem::Failed;
+    _freshTiles.push_back(failedItem);
+    _completedTiles[ failedItem._dir ] = now + UpdateInterval::FailedAttempt;
     _is_dirty = true;
 }
 
-#ifdef HAVE_SVN_CLIENT_H
-// Configure our subversion session
-int SGTerraSync::SvnThread::svnClientSetup(void)
+void SGTerraSync::SvnThread::updated(SyncItem item, bool isNewDirectory)
 {
-    // Are we already prepared?
-    if (_svn_pool) return EXIT_SUCCESS;
-    // No, so initialize svn internals generally
-
-#ifdef _MSC_VER
-    // there is a segfault when providing an error stream.
-    //  Apparently, calling setvbuf with a nul buffer is
-    //  not supported under msvc 7.1 ( code inside svn_cmdline_init )
-    if (svn_cmdline_init("terrasync", 0) != EXIT_SUCCESS)
-        return EXIT_FAILURE;
-
-    // revert locale setting
-    setlocale(LC_ALL,"C");
-#else
-    /* svn_cmdline_init configures the locale. Setup environment to ensure the
-     * default "C" locale remains active, since fgfs isn't locale aware - especially
-     * requires "." as decimal point in strings containing floating point varibales. */
-    setenv("LC_ALL", "C", 1);
-
-    if (svn_cmdline_init("terrasync", stderr) != EXIT_SUCCESS)
-        return EXIT_FAILURE;
-#endif
-
-    apr_pool_t *pool = NULL;
+    time_t now = time(0);
+    _consecutive_errors = 0;
+    _success_count++;
+    SG_LOG(SG_TERRAIN,SG_INFO,
+           "Successfully synchronized directory '" << item._dir << "'");
     
-    apr_allocator_t* allocator = NULL;
-    int aprErr = apr_allocator_create(&allocator);
-    if (aprErr != APR_SUCCESS)
-        return EXIT_FAILURE;
-    
-    apr_pool_create_ex(&pool, NULL /* parent pool */, NULL /* abort func */, allocator);
-    
-    svn_error_t *err = NULL;
-    SVN_VERSION_DEFINE(_svn_version);
-    err = svn_ver_check_list(&_svn_version, mysvn_checklist);
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
-    err = svn_ra_initialize(pool);
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
-    char *config_dir = NULL;
-    err = svn_config_ensure(config_dir, pool);
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
-    err = svn_client_create_context(&_svn_ctx, pool);
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
-    err = svn_config_get_config(&(_svn_ctx->config),
-        config_dir, pool);
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
-    svn_config_t *cfg;
-    cfg = ( svn_config_t*) apr_hash_get(
-        _svn_ctx->config,
-        SVN_CONFIG_CATEGORY_CONFIG,
-        APR_HASH_KEY_STRING);
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
+    item._status = SyncItem::Updated;
+    if (item._type == SyncItem::Tile) {
+        _updated_tile_count++;
+    }
 
-    svn_auth_baton_t *ab=NULL;
+    _freshTiles.push_back(item);
+    _completedTiles[ item._dir ] = now + UpdateInterval::SuccessfulAttempt;
+    _is_dirty = true;
+    writeCompletedTilesPersistentCache();
+}
 
-#if (SVN_VER_MINOR >= 6)
-    err = svn_cmdline_create_auth_baton  (&ab,
-            TRUE, NULL, NULL, config_dir, TRUE, FALSE, cfg,
-            _svn_ctx->cancel_func, _svn_ctx->cancel_baton, pool);
-#else
-    err = svn_cmdline_setup_auth_baton(&ab,
-        TRUE, NULL, NULL, config_dir, TRUE, cfg,
-        _svn_ctx->cancel_func, _svn_ctx->cancel_baton, pool);
-#endif
+void SGTerraSync::SvnThread::initCompletedTilesPersistentCache()
+{
+    if (!_persistentCachePath.exists()) {
+        return;
+    }
+    
+    SGPropertyNode_ptr cacheRoot(new SGPropertyNode);
+    time_t now = time(0);
+    
+    try {
+        readProperties(_persistentCachePath.str(), cacheRoot);
+    } catch (sg_exception& e) {
+        SG_LOG(SG_TERRAIN, SG_INFO, "corrupted persistent cache, discarding");
+        return;
+    }
+    
+    for (int i=0; i<cacheRoot->nChildren(); ++i) {
+        SGPropertyNode* entry = cacheRoot->getChild(i);
+        string tileName = entry->getStringValue("path");
+        time_t stamp = entry->getIntValue("stamp");
+        if (stamp < now) {
+            continue;
+        }
+        
+        _completedTiles[tileName] = stamp;
+    }
+}
 
-    if (err)
-        return svn_cmdline_handle_exit_error(err, pool, "fgfs: ");
+void SGTerraSync::SvnThread::writeCompletedTilesPersistentCache() const
+{
+    // cache is disabled
+    if (_persistentCachePath.isNull()) {
+        return;
+    }
     
-    _svn_ctx->auth_baton = ab;
-#if (SVN_VER_MINOR >= 5)
-    _svn_ctx->conflict_func = NULL;
-    _svn_ctx->conflict_baton = NULL;
-#endif
-
-    // Now our magic revisions
-    _svn_rev = (svn_opt_revision_t*) apr_palloc(pool, 
-        sizeof(svn_opt_revision_t));
-    if (!_svn_rev)
-        return EXIT_FAILURE;
-    _svn_rev_peg = (svn_opt_revision_t*) apr_palloc(pool, 
-        sizeof(svn_opt_revision_t));
-    if (!_svn_rev_peg)
-        return EXIT_FAILURE;
-    _svn_rev->kind = svn_opt_revision_head;
-    _svn_rev_peg->kind = svn_opt_revision_unspecified;
-    // Success if we got this far
-    _svn_pool = pool;
-    return EXIT_SUCCESS;
+    std::ofstream f(_persistentCachePath.c_str(), std::ios::trunc);
+    if (!f.is_open()) {
+        return;
+    }
+    
+    SGPropertyNode_ptr cacheRoot(new SGPropertyNode);
+    CompletedTiles::const_iterator it = _completedTiles.begin();
+    for (; it != _completedTiles.end(); ++it) {
+        SGPropertyNode* entry = cacheRoot->addChild("entry");
+        entry->setStringValue("path", it->first);
+        entry->setIntValue("stamp", it->second);
+    }
+    
+    writeProperties(f, cacheRoot, true /* write_all */);
+    f.close();
 }
-#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 // SGTerraSync ////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
-SGTerraSync::SGTerraSync(SGPropertyNode_ptr root) :
+SGTerraSync::SGTerraSync() :
     _svnThread(NULL),
     last_lat(NOWHERE),
     last_lon(NOWHERE),
-    _terraRoot(root->getNode("/sim/terrasync",true)),
-    _refreshCb(NULL),
-    _userCbData(NULL)
+    _bound(false),
+    _inited(false)
 {
     _svnThread = new SvnThread();
+    _log = new BufferedLogCallback(SG_TERRAIN, SG_INFO);
+    _log->truncateAt(255);
+    
+    sglog().addCallback(_log);
 }
 
 SGTerraSync::~SGTerraSync()
 {
-    _tiedProperties.Untie();
     delete _svnThread;
     _svnThread = NULL;
+    sglog().removeCallback(_log);
+    delete _log;
+     _tiedProperties.Untie();
+}
+
+void SGTerraSync::setRoot(SGPropertyNode_ptr root)
+{
+    _terraRoot = root->getNode("/sim/terrasync",true);
 }
 
 void SGTerraSync::init()
 {
-    _refreshDisplay = _terraRoot->getNode("refresh-display",true);
+    if (_inited) {
+        return;
+    }
+    
+    _inited = true;
+    
+    assert(_terraRoot);
     _terraRoot->setBoolValue("built-in-svn-available",svn_built_in_available);
+        
     reinit();
 }
 
+void SGTerraSync::shutdown()
+{
+     _svnThread->stop();
+}
+
 void SGTerraSync::reinit()
 {
     // do not reinit when enabled and we're already up and running
     if ((_terraRoot->getBoolValue("enabled",false))&&
          (_svnThread->_active && _svnThread->_running))
+    {
         return;
-
+    }
+    
     _svnThread->stop();
 
     if (_terraRoot->getBoolValue("enabled",false))
     {
         _svnThread->setSvnServer(_terraRoot->getStringValue("svn-server",""));
+        _svnThread->setSvnDataServer(_terraRoot->getStringValue("svn-data-server",""));
         _svnThread->setRsyncServer(_terraRoot->getStringValue("rsync-server",""));
         _svnThread->setLocalDir(_terraRoot->getStringValue("scenery-dir",""));
         _svnThread->setAllowedErrorCount(_terraRoot->getIntValue("max-errors",5));
-
-    #ifdef HAVE_SVN_CLIENT_H
         _svnThread->setUseBuiltin(_terraRoot->getBoolValue("use-built-in-svn",true));
-    #else
-        _terraRoot->setBoolValue("use-built-in-svn",false);
-    #endif
+        _svnThread->setCachePath(SGPath(_terraRoot->getStringValue("cache-path","")));
+        _svnThread->setCacheHits(_terraRoot->getIntValue("cache-hit", 0));
         _svnThread->setUseSvn(_terraRoot->getBoolValue("use-svn",true));
         _svnThread->setExtSvnUtility(_terraRoot->getStringValue("ext-svn-utility","svn"));
 
@@ -743,11 +847,23 @@ void SGTerraSync::reinit()
 
 void SGTerraSync::bind()
 {
+    if (_bound) {
+        return;
+    }
+    
+    _bound = true;
     _tiedProperties.Tie( _terraRoot->getNode("busy", true), (bool*) &_svnThread->_busy );
     _tiedProperties.Tie( _terraRoot->getNode("active", true), (bool*) &_svnThread->_active );
     _tiedProperties.Tie( _terraRoot->getNode("update-count", true), (int*) &_svnThread->_success_count );
     _tiedProperties.Tie( _terraRoot->getNode("error-count", true), (int*) &_svnThread->_fail_count );
     _tiedProperties.Tie( _terraRoot->getNode("tile-count", true), (int*) &_svnThread->_updated_tile_count );
+    _tiedProperties.Tie( _terraRoot->getNode("cache-hits", true), (int*) &_svnThread->_cache_hits );
+    _tiedProperties.Tie( _terraRoot->getNode("transfer-rate-bytes-sec", true), (int*) &_svnThread->_transfer_rate );
+    
+    // use kbytes here because propety doesn't support 64-bit and we might conceivably
+    // download more than 2G in a single session
+    _tiedProperties.Tie( _terraRoot->getNode("downloaded-kbytes", true), (int*) &_svnThread->_total_kb_downloaded );
+    
     _terraRoot->getNode("busy", true)->setAttribute(SGPropertyNode::WRITE,false);
     _terraRoot->getNode("active", true)->setAttribute(SGPropertyNode::WRITE,false);
     _terraRoot->getNode("update-count", true)->setAttribute(SGPropertyNode::WRITE,false);
@@ -765,6 +881,12 @@ void SGTerraSync::unbind()
 {
     _svnThread->stop();
     _tiedProperties.Untie();
+    _bound = false;
+    _inited = false;
+    
+    _terraRoot.clear();
+    _stalledNode.clear();
+    _cacheHits.clear();
 }
 
 void SGTerraSync::update(double)
@@ -788,51 +910,19 @@ void SGTerraSync::update(double)
             _stalledNode->setBoolValue(_svnThread->_stalled);
         }
 
-        if (!_refreshDisplay->getBoolValue())
-            return;
-
         while (_svnThread->hasNewTiles())
         {
-            WaitingTile next = _svnThread->getNewTile();
-            if (next._refreshScenery)
-            {
-                refreshScenery(_svnThread->getLocalDir(),next._dir);
+            SyncItem next = _svnThread->getNewTile();
+            
+            if ((next._type == SyncItem::Tile) || (next._type == SyncItem::AIData)) {
+                _activeTileDirs.erase(next._dir);
             }
-        }
-    }
-}
-
-void SGTerraSync::refreshScenery(SGPath path,const string& relativeDir)
-{
-    // find tiles to be refreshed
-    if (_refreshCb)
-    {
-        path.append(relativeDir);
-        if (path.exists())
-        {
-            simgear::Dir dir(path);
-            //TODO need to be smarter here. only update tiles which actually
-            // changed recently. May also be possible to use information from the
-            // built-in SVN client directly (instead of checking directory contents).
-            PathList tileList = dir.children(simgear::Dir::TYPE_FILE, ".stg");
-            for (unsigned int i=0; i<tileList.size(); ++i)
-            {
-                // reload scenery tile
-                long index = atoi(tileList[i].file().c_str());
-                _refreshCb(_userCbData, index);
-            }
-        }
+        } // of freshly synced items
     }
 }
 
 bool SGTerraSync::isIdle() {return _svnThread->isIdle();}
 
-void SGTerraSync::setTileRefreshCb(SGTerraSyncCallback refreshCb, void* userCbData)
-{
-    _refreshCb = refreshCb;
-    _userCbData = userCbData;
-}
-
 void SGTerraSync::syncAirportsModels()
 {
     static const char* bounds = "MZAJKL"; // airport sync order: K-L, A-J, M-Z
@@ -843,11 +933,12 @@ void SGTerraSync::syncAirportsModels()
         {
             ostringstream dir;
             dir << "Airports/" << synced_other;
-            WaitingTile w(dir.str(),false);
+            SyncItem w(dir.str(), SyncItem::AirportData);
             _svnThread->request( w );
         }
     }
-    WaitingTile w("Models",false);
+    
+    SyncItem w("Models", SyncItem::SharedModels);
     _svnThread->request( w );
 }
 
@@ -884,17 +975,27 @@ void SGTerraSync::syncArea( int lat, int lon )
         EW = 'e';
     }
 
-    const char* terrainobjects[3] = { "Terrain", "Objects",  0 };
-    bool refresh=true;
+    ostringstream dir;
+    dir << setfill('0')
+    << EW << setw(3) << abs(baselon) << NS << setw(2) << abs(baselat) << "/"
+    << EW << setw(3) << abs(lon)     << NS << setw(2) << abs(lat);
+    
+    syncAreaByPath(dir.str());
+}
+
+void SGTerraSync::syncAreaByPath(const std::string& aPath)
+{
+    const char* terrainobjects[3] = { "Terrain/", "Objects/",  0 };
     for (const char** tree = &terrainobjects[0]; *tree; tree++)
     {
-        ostringstream dir;
-        dir << *tree << "/" << setfill('0')
-            << EW << setw(3) << abs(baselon) << NS << setw(2) << abs(baselat) << "/"
-            << EW << setw(3) << abs(lon)     << NS << setw(2) << abs(lat);
-        WaitingTile w(dir.str(),refresh);
+        std::string dir = string(*tree) + aPath;
+        if (_activeTileDirs.find(dir) != _activeTileDirs.end()) {
+            continue;
+        }
+                
+        _activeTileDirs.insert(dir);
+        SyncItem w(dir, SyncItem::Tile);
         _svnThread->request( w );
-        refresh=false;
     }
 }
 
@@ -902,6 +1003,7 @@ void SGTerraSync::syncArea( int lat, int lon )
 void SGTerraSync::syncAreas( int lat, int lon, int lat_dir, int lon_dir )
 {
     if ( lat_dir == 0 && lon_dir == 0 ) {
+        
         // do surrounding 8 1x1 degree areas.
         for ( int i = lat - 1; i <= lat + 1; ++i ) {
             for ( int j = lon - 1; j <= lon + 1; ++j ) {
@@ -927,6 +1029,12 @@ void SGTerraSync::syncAreas( int lat, int lon, int lat_dir, int lon_dir )
     syncArea( lat, lon );
 }
 
+bool SGTerraSync::scheduleTile(const SGBucket& bucket)
+{
+    std::string basePath = bucket.gen_base_path();
+    syncAreaByPath(basePath);
+    return true;
+}
 
 bool SGTerraSync::schedulePosition(int lat, int lon)
 {
@@ -937,8 +1045,6 @@ bool SGTerraSync::schedulePosition(int lat, int lon)
     {
         if (_svnThread->_running)
         {
-            SG_LOG(SG_TERRAIN,SG_DEBUG, "Requesting scenery update for position " <<
-                                        lat << "," << lon);
             int lat_dir=0;
             int lon_dir=0;
             if ( last_lat != NOWHERE && last_lon != NOWHERE )
@@ -976,3 +1082,46 @@ bool SGTerraSync::schedulePosition(int lat, int lon)
 
     return Ok;
 }
+
+void SGTerraSync::reposition()
+{
+    last_lat = last_lon = NOWHERE;
+}
+
+bool SGTerraSync::isTileDirPending(const std::string& sceneryDir) const
+{
+    if (!_svnThread->_running) {
+        return false;
+    }
+    
+    const char* terrainobjects[3] = { "Terrain/", "Objects/",  0 };
+    for (const char** tree = &terrainobjects[0]; *tree; tree++) {
+        string s = *tree + sceneryDir;
+        if (_activeTileDirs.find(s) != _activeTileDirs.end()) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+void SGTerraSync::scheduleDataDir(const std::string& dataDir)
+{
+    if (_activeTileDirs.find(dataDir) != _activeTileDirs.end()) {
+        return;
+    }
+    
+    _activeTileDirs.insert(dataDir);
+    SyncItem w(dataDir, SyncItem::AIData);
+    _svnThread->request( w );
+
+}
+
+bool SGTerraSync::isDataDirPending(const std::string& dataDir) const
+{
+    if (!_svnThread->_running) {
+        return false;
+    }
+    
+    return (_activeTileDirs.find(dataDir) != _activeTileDirs.end());
+}
